# -*- coding: utf-8 -*-
from gi.repository import Gtk,GObject

# Convenience library for a new kind of UI -- for e.g. this game, we
# don't really want to have dialogs. Nonetheless, it's convenient to
# design dialogs in gtkbuilder and run them in the standard manner... So we
# create a new "dialog" interface via a hidden notebook.

class SwappableArea (Gtk.Notebook):


    swallowed = {}
    response = None
    running = False


    def __init__ (self, main_area):
        GObject.GObject.__init__(self)
        self.set_show_tabs(False)
        self.set_show_border(False)
        self.main_page = self.append_page(main_area, None)

    def swallow_window (self, d):
        child = d.get_child()
        d.remove(child)
        return self.swallow_widget(child)

    def swallow_widget (self, w):
        w.unparent()
        return self.append_page(w, None)

    def response_cb (self, w, response, data=None):
        Gtk.main_quit()
        self.response = response

    def swallow_dialog (self, d):
        n = self.swallow_window(d)
        self.swallowed[d] = n
        self.set_current_page(n)
        d.connect('response', self.response_cb)

    def run_dialog (self, d):
        self.running = d
        if not self.swallowed.has_key(d):
            self.swallow_dialog(d)
        self.set_current_page(self.swallowed[d])
        try:
            Gtk.main()
        except:
            print 'Error in dialog!'
            import traceback
            traceback.print_exc()
            print 'forge on fearlessly...'
        self.set_current_page(self.main_page)
        self.running = None
        tmp_response = self.response
        self.response = None
        return tmp_response



if __name__ == '__main__':

    d = Gtk.Dialog()
    d.vbox.add(Gtk.Label('Foo, bar, baz'))
    d.vbox.show_all()
    d.add_button(Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE)
    w = Gtk.Window()
    b = Gtk.Button('show d')
    b.show()
    sa = SwappableArea(b)
    sa.show()
    w.add(sa)
    b.connect_object('clicked', sa.run_dialog, d)
    w.show()
    Gtk.main()
    



# -*- coding: utf-8 -*-
from gi.repository import Gtk,Gdk,GdkPixbuf
import cairo

SUDOKU_SIZE = 9
N_BOXES = 3

PENCIL_GREY = (0.3, 0.3, 0.3)
BACKGROUND_COLOR = (1., 1., 1.)

def draw_sudoku (cr, sudoku, played, size, offset_x = 0, offset_y = 0, border_color = (1.0, 1.0, 1.0), line_color = (0.0, 0.0, 0.0), for_printing = False):

    if for_printing:
        THIN = size / 300.
    else:
        THIN = 1

    THICK = THIN * 2.
    BORDER = THICK

    WHITE_SPACE = (size
                   - 2 * BORDER
                   - 2 * THICK
                   - (N_BOXES - 1) * THICK
                   - (N_BOXES * 2) * THIN
                  )

    SQUARE_SIZE = WHITE_SPACE / SUDOKU_SIZE

    if for_printing:
        FONT_SIZE = SQUARE_SIZE / 2
        FONT_WEIGHT = cairo.FONT_WEIGHT_NORMAL
    else:
        FONT_SIZE = SQUARE_SIZE
        FONT_WEIGHT = cairo.FONT_WEIGHT_BOLD

    outer = {}
    outer["left"]   = offset_x
    outer["right"]  = offset_x + size
    outer["top"]    = offset_y
    outer["bottom"] = offset_y + size

    # Entire background
    cr.set_source_rgb(1., 1., 1.)
    cr.rectangle(outer["left"],
                 outer["top"],
                 size,
                 size)
    cr.fill()

    # Outer border
    cr.set_line_join(cairo.LINE_JOIN_ROUND)
    cr.set_line_width(BORDER)
    cr.rectangle(outer["left"]   + BORDER/2.0,
                 outer["top"]    + BORDER/2.0,
                 size            - BORDER,
                 size            - BORDER)

    # Inner background
    cr.set_source_rgb(*BACKGROUND_COLOR)
    cr.fill_preserve()
    #Border box
    cr.set_source_rgb(*border_color)
    cr.stroke()

    #Outer thick lines
    cr.set_line_join(cairo.LINE_JOIN_MITER)
    cr.set_line_width(THICK)
    cr.rectangle(outer["left"]   + BORDER +   THICK/2.0,
                 outer["top"]    + BORDER +   THICK/2.0,
                 size            - BORDER * 2 - THICK,
                 size            - BORDER * 2 - THICK)

    cr.set_source_rgb(*line_color)
    cr.stroke()

    inner = {}
    inner["left"]   = outer["left"]   + BORDER + THICK
    inner["right"]  = outer["right"]  - BORDER - THICK
    inner["top"]    = outer["top"]    + BORDER + THICK
    inner["bottom"] = outer["bottom"] - BORDER - THICK

    pos = {}
    position = BORDER + THICK
    pos[0] = position + SQUARE_SIZE/2.
    last_line = 0
    for n in range(1, SUDOKU_SIZE):
        if n % N_BOXES == 0:
            cr.set_line_width(THICK)
            position += SQUARE_SIZE + last_line/2.0 + THICK/2.0
            last_line = THICK
        else:
            cr.set_line_width(THIN)
            position += SQUARE_SIZE + last_line/2.0 + THIN/2.0
            last_line = THIN
        pos[n] = position + last_line/2. + SQUARE_SIZE/2.0
        cr.move_to(BORDER + THICK/2. + offset_x, position + offset_y)
        cr.line_to(size - BORDER - THICK/2. + offset_x, position + offset_y)
        cr.move_to(position + offset_x, BORDER + THICK/2. + offset_y)
        cr.line_to(position + offset_x, size - BORDER - THICK/2. + offset_y)
        cr.stroke()
    cr.set_font_size(FONT_SIZE)
    for x in range(SUDOKU_SIZE):
        for y in range(SUDOKU_SIZE):
            cr.move_to(pos[x] + offset_x, pos[y] + offset_y)
            letter = None
            if sudoku and sudoku[y][x]:
                letter = str(sudoku[y][x])
                cr.select_font_face("",
                         cairo.FONT_SLANT_NORMAL,
                         FONT_WEIGHT)
                cr.set_source_rgb(0, 0, 0)
                xbearing, ybearing, width, height, xadvance, yadvance = (
                    cr.text_extents(letter)
                    )

            elif played and played[y][x]:
                cr.select_font_face("Times",
                                    cairo.FONT_SLANT_ITALIC,
                                    cairo.FONT_WEIGHT_NORMAL)
                cr.set_source_rgb(*PENCIL_GREY)
                letter = str(played[y][x])
                xbearing, ybearing, width, height, xadvance, yadvance = (
                    cr.text_extents(letter)
                    )
            if letter:
                cr.move_to(pos[x] + offset_x - (xadvance/2.0),
                           pos[y] + offset_y + (height/2.0))
                cr.show_text(letter)

def make_pixbuf (sudoku, played, border_color, line_color = (0.4, 0.4, 0.4)):
    size = 126
    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, size, size)
    cr = cairo.Context(surface)
    draw_sudoku(cr, sudoku, played,  size, 0, 0, border_color, line_color)
    pixbuf = Gdk.pixbuf_get_from_surface(surface, 0, 0, surface.get_width(), surface.get_height())
    del surface
    return pixbuf

if __name__ == "__main__":
    sudoku = [[0, 0, 2, 3, 4, 5, 6, 0, 0]] * SUDOKU_SIZE
    played = [[1, 2, 0, 0, 0, 0, 0, 0, 0, 0]] + [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0]] * 8


    size = 250
    line_color = (0.0, 0.0, 0.0)
    border_color = (1.0, 0.0, 0.0)
    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 750, 750)
    cr = cairo.Context(surface)
    draw_sudoku(cr, sudoku, played,  size, 100, 250, border_color, line_color)
    pb = Gdk.pixbuf_get_from_surface(surface, 0, 0, surface.get_width(), surface.get_height())
    del surface


    w = Gtk.Window()
    img = Gtk.Image()
    img.set_from_pixbuf(pb)
    w.add(img)
    w.show_all()
    Gtk.main()




# -*- coding: utf-8 -*-
#!/usr/bin/python

from gi.repository import Gtk,Gdk,GObject,Pango,PangoCairo
import math
import random
import tracker_info
from gettext import gettext as _

from simple_debug import simple_debug

ERROR_HIGHLIGHT_COLOR = (1.0, 0, 0)

BASE_SIZE = 35 # The "normal" size of a box (in pixels)

# And the standard font-sizes -- these should fit nicely with the
# BASE_SIZE
BASE_FONT_SIZE = Pango.SCALE * 13
NOTE_FONT_SIZE = Pango.SCALE * 6

BORDER_WIDTH = 9.0 # The size of space we leave for a box
NORMAL_LINE_WIDTH = 1 # The size of the line we draw around a box

DEBUG_COLORS = False

def debug_set_color_rgba(cr, rgba):
    COLORS = ("red","green","blue","yellow","purple","wheat","maroon","gray")

    if DEBUG_COLORS:
        rgba = Gdk.RGBA()
        rgba.parse(COLORS[random.randint(0,len(COLORS)-1)])

    Gdk.cairo_set_source_rgba(cr, rgba)

class NumberSelector (Gtk.EventBox):

    __gsignals__ = {
        'changed':(GObject.SignalFlags.RUN_LAST, None, ()),
        }

    def __init__ (self, default = None, upper = 9):
        self.value = default
        GObject.GObject.__init__(self)
        self.table = Gtk.Table()
        self.add(self.table)
        side = int(math.sqrt(upper))
        n = 1
        for y in range(side):
            for x in range(side):
                b = Gtk.Button()
                l = Gtk.Label()
                if n == self.value:
                    l.set_markup('<b><span size="x-small">%s</span></b>'%n)
                else:
                    l.set_markup('<span size="x-small">%s</span>'%n)
                b.add(l)
                b.set_relief(Gtk.ReliefStyle.HALF)
                l = b.get_children()[0]
                b.set_border_width(0)
                l.set_padding(0, 0)
                l.get_alignment()
                b.connect('clicked', self.number_clicked, n)
                self.table.attach(b, x, x+1, y, y+1)
                n += 1
        if self.value:
            db = Gtk.Button()
            l = Gtk.Label()
            l.set_markup_with_mnemonic('<span size="x-small">'+_('_Clear')+'</span>')
            db.add(l)
            l.show()
            db.connect('clicked', self.number_clicked, 0)
            self.table.attach(db, 0, side, side + 1, side + 2)
        self.show_all()

    def number_clicked (self, button, n):
        self.value = n
        self.emit('changed')

    def get_value (self):
        return self.value

    def set_value (self, n):
        self.value = n

class NumberBox (Gtk.DrawingArea):

    text = ''
    top_note_text = ''
    bottom_note_text = ''
    read_only = False
    _layout = None
    _top_note_layout = None
    _bottom_note_layout = None
    text_color = None
    highlight_color = None
    shadow_color = None
    custom_background_color = None
    border_color = None

    __gsignals__ = {
        'value-about-to-change':(GObject.SignalFlags.RUN_LAST, None, ()),
        'notes-about-to-change':(GObject.SignalFlags.RUN_LAST, None, ()),
        'changed':(GObject.SignalFlags.RUN_LAST, None, ()),
        # undo-change - A hacky way to handle the fact that we want to
        # respond to undo's changes but we don't want undo to respond
        # to itself...
        'undo-change':(GObject.SignalFlags.RUN_LAST, None, ()),
        'notes-changed':(GObject.SignalFlags.RUN_LAST, None, ()),
        }

    base_state = Gtk.StateFlags.NORMAL
    npicker = None
    draw_boxes = False

    def __init__ (self, upper = 9, text = ''):
        Gtk.DrawingArea.__init__(self)
        self.upper = upper
        self.parent_win = None
        self.timer = None
        self.font = self.get_style().font_desc
        self.font.set_size(BASE_FONT_SIZE)
        self.note_font = self.font.copy()
        self.note_font.set_size(NOTE_FONT_SIZE)
        self._top_note_layout = Pango.Layout(self.create_pango_context())
        self._top_note_layout.set_font_description(self.note_font)
        self._bottom_note_layout = Pango.Layout(self.create_pango_context())
        self._bottom_note_layout.set_font_description(self.note_font)
        self._base_stateflags = Gtk.StateFlags.NORMAL
        self.top_note_list = []
        self.bottom_note_list = []
        self.tinfo = tracker_info.TrackerInfo()
        self.set_property('can-focus', True)
        self.set_property('events', Gdk.EventMask.ALL_EVENTS_MASK)
        self.connect('button-press-event', self.button_press_cb)
        self.connect('key-release-event', self.key_press_cb)
        self.connect('enter-notify-event', self.pointer_enter_cb)
        self.connect('leave-notify-event', self.pointer_leave_cb)
        self.connect('focus-in-event', self.focus_in_cb)
        self.connect('focus-out-event', self.focus_out_cb)
        self.connect('motion-notify-event', self.motion_notify_cb)
        self.set_text(text)

    def set_parent_win(self, new_parent):
        self.parent_win = new_parent

    def set_timer(self, new_timer):
        self.timer = new_timer

    def pointer_enter_cb (self, *args):
        if not self.is_focus():
            self.set_state_flags(Gtk.StateFlags.PRELIGHT, False)

    def pointer_leave_cb (self, *args):
        self.set_state_flags(self._base_stateflags, True)
        self._toggle_box_drawing_(False)

    def focus_in_cb (self, *args):
        self.set_state_flags(Gtk.StateFlags.SELECTED, True)
        self._base_stateflags = Gtk.StateFlags.SELECTED

    def focus_out_cb (self, *args):
        self.set_state_flags(Gtk.StateFlags.NORMAL, True)
        self._base_stateflags = Gtk.StateFlags.NORMAL
        self.destroy_npicker()

    def destroy_npicker (self):
        if self.npicker:
            self.npicker.destroy()
            self.npicker = None

    def motion_notify_cb (self, *args):
        if self.is_focus() and not self.read_only:
            self._toggle_box_drawing_(True)
        else:
            self._toggle_box_drawing_(False)

    def _toggle_box_drawing_ (self, val):
        if val and not self.draw_boxes:
            self.draw_boxes = True
            self.queue_draw()
        if (not val) and self.draw_boxes:
            self.draw_boxes = False
            self.queue_draw()

    def button_press_cb (self, w, e):
        if self.read_only:
            return
        if e.type == Gdk.EventType._2BUTTON_PRESS:
            # ignore second click (this makes a double click in the
            # middle of a cell get us a display of the numbers, rather
            # than selecting a number.
            return
        if self.is_focus():
            x, y = e.get_coords()
            alloc = self.get_allocation()
            my_w = alloc.width
            my_h = alloc.height
            border_height = float(BORDER_WIDTH)/BASE_SIZE

            if float(y)/my_h < border_height:
                self.show_note_editor(top = True)
            elif float(y)/my_h > (1-border_height):
                self.show_note_editor(top = False)
            elif not self.npicker:
                # In this case we're a normal old click...
                # makes sure there is only one numer selector
                self.show_number_picker()
        else:
            self.grab_focus()

    def key_press_cb (self, w, e):
        if self.read_only:
            return
        if self.npicker: # kill number picker no matter what is pressed
            self.destroy_npicker()
        txt = Gdk.keyval_name(e.keyval)
        if type(txt) == type(None):
            # Make sure we don't trigger on unplugging the A/C charger etc
            return
        txt = txt.replace('KP_', '')

        # Add the new value if need be
        if txt in [str(n) for n in range(1, self.upper+1)]:
            if e.state & Gdk.ModifierType.CONTROL_MASK:
                self.add_note_text(txt, top = True)
            elif e.state & Gdk.ModifierType.MOD1_MASK:
                self.remove_note_text(txt, top = True)
            elif self.get_text() != txt or \
                (self.tracker_id != tracker_info.NO_TRACKER and
                 self.tinfo.current_tracker == tracker_info.NO_TRACKER):
                # If there's no change, do nothing unless the player wants to
                # change a tracked item while not tracking(ie commit a tracked
                # change)
                self.set_text_interactive(txt)
        elif txt in ['0', 'Delete', 'BackSpace']:
            self.set_text_interactive('')
        elif txt in ['n', 'N']:
            if e.state & Gdk.ModifierType.MOD1_MASK:
                self.set_note_text_interactive(top_text = '')
            else:
                self.show_note_editor(top = True)
        elif txt in ['m', 'M']:
            if e.state & Gdk.ModifierType.MOD1_MASK:
                self.set_note_text_interactive(bottom_text = '')
            else:
                self.show_note_editor(top = False)

    def add_note_text(self, txt, top = False):
        if top:
            note = self.top_note_text
        else:
            note = self.bottom_note_text
        if txt not in note:
            tmp = list(note)
            tmp.append(txt)
            tmp.sort()
            note = ''.join(tmp)
            if top:
                self.set_note_text_interactive(top_text = note)
            else:
                self.set_note_text_interactive(bottom_text = note)

    def remove_note_text(self, txt, top = False):
        if top:
            note = self.top_note_text
        else:
            note = self.bottom_note_text
        if txt in note:
            note = note.replace(txt,'')
            if top:
                self.set_note_text_interactive(top_text = note)
            else:
                self.set_note_text_interactive(bottom_text = note)

    def note_changed_cb (self, w, top = False):
        if top:
            self.set_note_text_interactive(top_text = w.get_text())
        else:
            self.set_note_text_interactive(bottom_text = w.get_text())

    def note_focus_in(self, win, evt):
        if (self.timer):
            self.timer.resume_timing()

    def note_focus_out(self, wgt, evt):
        if (self.timer):
            self.timer.pause_timing()

    def show_note_editor (self, top = True):
        alloc = self.get_allocation()
        w = Gtk.Window()
        w.set_property('skip-pager-hint', True)
        w.set_property('skip-taskbar-hint', True)
        w.set_decorated(False)
        w.set_position(Gtk.WindowPosition.MOUSE)
        w.set_size_request(alloc.width, alloc.height/2)
        if self.parent_win:
            w.set_transient_for(self.parent_win)
        f = Gtk.Frame()
        e = Gtk.Entry()
        f.add(e)
        if top:
            e.set_text(self.top_note_text)
        else:
            e.set_text(self.bottom_note_text)
        w.add(f)
        e.connect('changed', self.note_changed_cb, top)
        e.connect('focus-in-event', self.note_focus_in)
        e.connect('focus-out-event', lambda e, ev, w: w.destroy(), w)
        e.connect('focus-out-event', self.note_focus_out)
        e.connect('activate', lambda e, w: w.destroy(), w)
        _, x, y = self.get_window().get_origin()
        if top:
            w.move(x, y)
        else:
            w.move(x, y+int(alloc.height*0.6))
        w.show_all()
        e.grab_focus()

    def number_changed_cb (self, num_selector):
        self.destroy_npicker()
        newval = num_selector.get_value()
        if newval:
            self.set_text_interactive(str(newval))
        else:
            self.set_text_interactive('')            

    def show_number_picker (self):
        w = Gtk.Window(type = Gtk.WindowType.POPUP)
        ns = NumberSelector(upper = self.upper, default = self.get_value())
        ns.connect('changed', self.number_changed_cb)
        w.grab_focus()
        w.add(ns)
        _, xorigin, yorigin = self.get_window().get_origin()
        x, y = (self.get_allocated_width(), self.get_allocated_height())
        popupx, popupy = w.get_size()
        overlapx = popupx-x
        overlapy = popupy-y
        w.move(xorigin - (overlapx/2), yorigin - (overlapy/2))
        w.show()
        self.npicker = w

    def set_text_interactive (self, text):
        self.emit('value-about-to-change')
        self.set_text(text)
        self.queue_draw()
        self.emit('changed')

    def set_font (self, font):
        if type(font) == str:
            font = Pango.FontDescription(font)
        self.font = font
        if self.text:
            self.set_text(self.text)
        self.queue_draw()

    def set_note_font (self, font):
        if type(font) == str:
            font = Pango.FontDescription(font)
        self.note_font = font
        self._top_note_layout.set_font_description(font)
        self._bottom_note_layout.set_font_description(font)
        self.queue_draw()

    def set_text (self, text):
        self.text = text
        self._layout = self.create_pango_layout(text)
        self._layout.set_font_description(self.font)

    def show_note_text (self):
        '''Display the notes for the current view
        '''
        self.top_note_text = self.get_note_display(self.top_note_list)[1]
        self._top_note_layout.set_markup(self.get_note_display(self.top_note_list)[2], -1)
        self.bottom_note_text = self.get_note_display(self.bottom_note_list)[1]
        self._bottom_note_layout.set_markup(self.get_note_display(self.bottom_note_list)[2], -1)
        self.queue_draw()

    def set_note_text (self, top_text = None, bottom_text = None, for_hint = False):
        '''Change the notes
        '''
        if top_text is not None:
            self.update_notelist(self.top_note_list, top_text)
        if bottom_text is not None:
            self.update_notelist(self.bottom_note_list, bottom_text, for_hint)
        self.show_note_text()

    def set_note_text_interactive (self, *args, **kwargs):
        self.emit('notes-about-to-change')
        self.set_note_text(*args, **kwargs)
        self.emit('notes-changed')

    def set_notelist(self, top_notelist, bottom_notelist):
        '''Assign new note lists
        '''
        if top_notelist:
            self.top_note_list = top_notelist
        if bottom_notelist:
            self.bottom_note_list = bottom_notelist

    def get_note_display(self, notelist, tracker_id = None, include_untracked = True):
        '''Parse a notelist for display

        Parse a notelist for the display.
        notelist - This method works on one notelist at a time, so
            top_note_list or bottom_note_list must be passed in.
        tracker_id - can specify a particular tracker.  The default is to use
            tracker that is currently showing.
        include_untracked - When set to True(default), the untracked notes will
            be included in the output.  Set it to false to exclude untracked
            notes.

        The output is returned in 3 formats:
        display_list - is tuple list in the format (notelist_index, tid, note)
            notelist_index - the index within the notelist
            tid - tracker id
            note - value of the note
        display_text - vanilla string representing all the values
        markup_text - pango markup string that colors each note for its tracker
        '''
        display_list = []
        display_text = ''
        markup_text = ''
        if tracker_id == None:
            tracker_id = self.tinfo.showing_tracker
        if include_untracked:
            track_filter = [tracker_info.NO_TRACKER, tracker_id]
        else:
            track_filter = [tracker_id]
        last_tracker = tracker_info.NO_TRACKER
        for notelist_index, (tid, note) in enumerate(notelist[:]):
            if tid not in track_filter:
                continue
            display_list.append((notelist_index, tid, note))
            display_text += note
            if tid != last_tracker:
                if self.tinfo.get_color_markup(last_tracker):
                    markup_text += '</span>'
                if self.tinfo.get_color_markup(tid):
                    markup_text += '<span foreground="' + str(self.tinfo.get_color_markup(tid)) + '">'
                last_tracker = tid
            markup_text += note
        if self.tinfo.get_color_markup(last_tracker):
            markup_text += '</span>'
        return((display_list, display_text, markup_text))

    def update_notelist(self, notelist, new_notes, for_hint = False):
        '''Parse notes for a notelist

        A notelist stores individual notes in the format (tracker, note).  The
        sequence is also meaningful - it dictates the order in which the notes
        are displayed.  One notelist is maintained for the top
        notes(top_note_list), and one for the bottom(bottom_note_list).  This
        method is responsible for maintaining those lists.

        When updating for hints(for_hint == True), the old notes are replaced
        completely by the new notes and set with NO_TRACKER.
        '''
        # Remove any duplicates
        unique_notes = ""
        for note in new_notes:
            if note not in unique_notes:
                unique_notes += note
        # Create a list and text version of the notelist
        display_list = self.get_note_display(notelist)[0]
        display_text = self.get_note_display(notelist)[1]
        if display_text == unique_notes:
            return
        # Remove deleted values from the notelist
        del_offset = 0
        for display_index, (notelist_index, tid, old_note) in enumerate(display_list[:]):
            if old_note not in unique_notes or for_hint:
                del notelist[notelist_index + del_offset]
                del display_list[display_index + del_offset]
                del_offset -= 1
            else:
                # Adjust the display_list index
                display_list[display_index + del_offset] = (notelist_index + del_offset, tid, old_note)
        # Insert any new values into the notelist
        ins_offset = 0
        display_index = 0
        for new_index, new_note in enumerate(unique_notes):
            add_note = False
            # if the new notes are longer than the current ones - append
            if len(display_list) <= display_index:
                notelist_index = len(notelist)
                ins_offset = 0
                add_note = True
            # Otherwise - advance until we find the appropriate place to insert
            else:
                old_note = display_list[display_index][2]
                if new_note != old_note:
                    notelist_index = display_list[display_index][0]
                    add_note = True
                display_index += 1
            if add_note:
                if for_hint:
                    use_tracker = tracker_info.NO_TRACKER
                else:
                    use_tracker = self.tinfo.current_tracker
                notelist.insert(notelist_index + ins_offset, (use_tracker, new_note))
                display_list.insert(new_index, (notelist_index + ins_offset, self.tinfo.current_tracker, new_note))
                ins_offset = ins_offset + 1
        self.trim_untracked_notes(notelist)

    def trim_untracked_notes(self, notelist):
        untracked_text = self.get_note_display(notelist, tracker_info.NO_TRACKER)[1]
        for tid, note in notelist[:]:
            if note in untracked_text and tid != tracker_info.NO_TRACKER:
                notelist.remove((tid, note))

    def get_notes_for_undo(self):
        '''Return the top and bottom notelists
        '''
        return((self.top_note_list[:], self.bottom_note_list[:]))

    def set_notes_for_undo(self, notelists):
        '''Reset the top and bottom notelists from an undo
        '''
        self.top_note_list, self.bottom_note_list = notelists
        self.show_note_text()

    @simple_debug
    def do_draw(self, cr):

        w = self.get_allocated_width()
        h = self.get_allocated_height()
        style_ctx = self.get_style_context()

        self.draw_background_color(cr, style_ctx, w, h)
        if self.is_focus():
            self.draw_highlight_box(cr, style_ctx, w, h)
        if self.border_color is not None:
            border_width = 3.0
            cr.set_source_rgb(*self.border_color)
            cr.rectangle(border_width*0.5, border_width*0.5, w-border_width, h-border_width)
            cr.set_line_width(border_width)
            cr.stroke()

        if h < w:
            scale = h/float(BASE_SIZE)
        else:
            scale = w/float(BASE_SIZE)
        cr.scale(scale, scale)

        self.draw_text(cr, style_ctx)
        if self.draw_boxes and self.is_focus():
            self.draw_note_area_highlight_box(cr, style_ctx)

    def draw_background_color (self, cr, style_ctx, w, h):
        if self.read_only:
            if self.custom_background_color:
                r, g, b = self.custom_background_color
                cr.set_source_rgb(
                    r*0.6, g*0.6, b*0.6
                    )
            else:
                #cr.set_source_color(self.style.base[Gtk.StateFlags.INSENSITIVE])
                #Gdk.cairo_set_source_rgba(
                debug_set_color_rgba(
                        cr, style_ctx.get_color(Gtk.StateFlags.INSENSITIVE))
        elif self.is_focus():
            #cr.set_source_color(self.style.base[Gtk.StateFlags.SELECTED])
            #Gdk.cairo_set_source_rgba(
            debug_set_color_rgba(
                    cr, style_ctx.get_color(Gtk.StateFlags.SELECTED))
        elif self.custom_background_color:
            cr.set_source_rgb(*self.custom_background_color)
        else:
            #cr.set_source_color(
            #    self.style.base[self.state]
            #    )
            #Gdk.cairo_set_source_rgba(
            cr.set_source_rgb(1.0, 1.0, 1.0)
        cr.rectangle(
            0, 0, w, h,
            )
        cr.fill()

    def draw_highlight_box (self, cr, style_ctx, w, h):
        #cr.set_source_color(
        #    self.style.base[Gtk.StateFlags.SELECTED]
        #    )
        #Gdk.cairo_set_source_rgba(
        debug_set_color_rgba(
                    cr, style_ctx.get_background_color(Gtk.StateFlags.SELECTED))

        border = 4 * w / BASE_SIZE
        cr.rectangle(
            # left-top
            border*0.5,
            border*0.5,
            # bottom-right
            w-border,
            h-border,
            )
        cr.set_line_width(border)
        cr.stroke()

    def draw_note_area_highlight_box (self, cr, style_ctx):
        # set up our paint brush...
        #cr.set_source_color(
        #    self.style.mid[self.state]
        #    )
        #Gdk.cairo_set_source_rgba(
        debug_set_color_rgba(
                cr, style_ctx.get_border_color(self.get_state_flags()))

        cr.set_line_width(NORMAL_LINE_WIDTH)
        # top rectangle
        cr.rectangle(NORMAL_LINE_WIDTH*0.5,
                     NORMAL_LINE_WIDTH*0.5,
                     BASE_SIZE-NORMAL_LINE_WIDTH,
                     BORDER_WIDTH-NORMAL_LINE_WIDTH)
        cr.stroke()
        # bottom rectangle
        cr.rectangle(NORMAL_LINE_WIDTH*0.5, #x
                     BASE_SIZE - BORDER_WIDTH-(NORMAL_LINE_WIDTH*0.5), #y
                     BASE_SIZE-NORMAL_LINE_WIDTH, #x2
                     BASE_SIZE-NORMAL_LINE_WIDTH #y2
                     )
        cr.stroke()

    def draw_text (self, cr, style_ctx):
        fontw, fonth = self._layout.get_pixel_size()
        # Draw a shadow for tracked conflicts.  This is done to
        # differentiate between tracked and untracked conflicts.
        if self.shadow_color:
            cr.set_source_rgb(*self.shadow_color)
            for xoff, yoff in [(1,1),(2,2)]:
                cr.move_to((BASE_SIZE/2)-(fontw/2) + xoff, (BASE_SIZE/2) - (fonth/2) + yoff)
                PangoCairo.show_layout(cr, self._layout)
        if self.text_color:
            cr.set_source_rgb(*self.text_color)
        elif self.read_only:
            #cr.set_source_color(self.style.text[Gtk.StateFlags.NORMAL])
            #Gdk.cairo_set_source_rgba(
            debug_set_color_rgba(
                cr, style_ctx.get_color(Gtk.StateFlags.NORMAL))
        else:
            #cr.set_source_color(self.style.text[self.state])
            #Gdk.cairo_set_source_rgba(
            debug_set_color_rgba(
                cr, style_ctx.get_color(Gtk.StateFlags.NORMAL))

        # And draw the text in the middle of the allocated space
        if self._layout:
            cr.move_to(
                (BASE_SIZE/2)-(fontw/2),
                (BASE_SIZE/2) - (fonth/2),
                )
            PangoCairo.update_layout(cr, self._layout)
            PangoCairo.show_layout(cr, self._layout)

        #cr.set_source_color(self.style.text[self.state])
        #Gdk.cairo_set_source_rgba(
        debug_set_color_rgba(
            cr, style_ctx.get_color(Gtk.StateFlags.NORMAL))

        # And draw any note text...
        if self._top_note_layout:
            fontw, fonth = self._top_note_layout.get_pixel_size()
            cr.move_to(
                NORMAL_LINE_WIDTH,
                0,
                )
            PangoCairo.update_layout(cr, self._top_note_layout)
            PangoCairo.show_layout(cr, self._top_note_layout)
        if self._bottom_note_layout:
            fontw, fonth = self._bottom_note_layout.get_pixel_size()
            cr.move_to(
                NORMAL_LINE_WIDTH,
                BASE_SIZE-fonth,
                )
            PangoCairo.update_layout(cr, self._bottom_note_layout)
            PangoCairo.show_layout(cr, self._bottom_note_layout)

    def set_text_color (self, color, shadow = None):
        self.shadow_color = shadow
        self.text_color = color
        self.queue_draw()

    def set_background_color (self, color):
        self.custom_background_color = color
        self.queue_draw()

    def set_border_color (self, color):
        self.border_color = color
        self.queue_draw()

    def hide_notes (self):
        pass

    def show_notes (self):
        pass

    def set_value (self, v):
        if 0 < v <= self.upper:
            self.set_text(str(v))
        else:
            self.set_text('')
        self.queue_draw()

    def get_value (self):
        try:
            return int(self.text)
        except:
            return None

    def get_text (self):
        return self.text

    def get_note_text (self):
        return self.top_note_text, self.bottom_note_text

class SudokuNumberBox (NumberBox):

    normal_color = None
    tracker_id = None
    error_color = (1.0, 0, 0)
    highlight_color = ERROR_HIGHLIGHT_COLOR

    def set_value(self, val, tracker_id = None):
        if tracker_id == None:
            self.tracker_id = self.tinfo.current_tracker
        else:
            self.tracker_id = tracker_id
        self.normal_color = self.tinfo.get_color(self.tracker_id)
        self.set_text_color(self.normal_color)
        super(SudokuNumberBox, self).set_value(val)

    def get_value_for_undo(self):
        return(self.tracker_id, self.get_value(), self.tinfo.get_trackers_for_cell(self.x, self.y))

    def set_value_for_undo (self, undo_val):
        tracker_id, value, all_traces = undo_val
        # When undo sets a value, switch to that tracker
        if value:
            self.tinfo.ui.select_tracker(tracker_id)
        self.set_value(value, tracker_id)
        self.tinfo.reset_trackers_for_cell(self.x, self.y, all_traces)
        self.emit('undo_change')

    def recolor(self, tracker_id):
        self.normal_color = self.tinfo.get_color(tracker_id)
        self.set_text_color(self.normal_color)

    def set_error_highlight (self, val):
        if val:
            if (self.tracker_id != tracker_info.NO_TRACKER):
                self.set_text_color(self.error_color, self.normal_color)
            else:
                self.set_text_color(self.error_color)
        else:
            self.set_text_color(self.normal_color)

    def set_read_only (self, val):
        self.read_only = val
        if not hasattr(self, 'bold_font'):
            self.normal_font = self.font
            self.bold_font = self.font.copy()
            self.bold_font.set_weight(Pango.Weight.BOLD)
        if self.read_only:
            self.set_font(self.bold_font)
        else:
            self.set_font(self.normal_font)
        self.queue_draw()

    def set_impossible (self, val):
        if val:
            if not self.get_text():
                self.set_text('X')
                self.set_text_color(self.error_color)
        elif self.get_text() == 'X':
            self.set_text('')
            self.set_text_color(self.normal_color)
        self.queue_draw()


GObject.type_register(NumberBox)

if __name__ == '__main__':
    window = Gtk.Window()
    window.connect('delete-event', Gtk.main_quit)

    def test_number_selector ():
        nselector = NumberSelector(default = 3)
        def tell_me (b):
            print 'value->', b.get_value()
        nselector.connect('changed', tell_me)
        window.add(nselector)

    def test_number_box ():
        window.set_size_request(100, 100)
        nbox = NumberBox()
        window.add(nbox)

#    test_number_selector()
    test_number_box()
    window.show_all()
    Gtk.main()



# -*- coding: utf-8 -*-
from gi.repository import Gtk,GdkPixbuf,GObject
import os.path
import sudoku, saver, sudoku_maker
import sudoku_thumber
from gettext import gettext as _
from gettext import ngettext
import time
import defaults
from simple_debug import simple_debug
from colors import color_hex_to_float

def color_from_difficulty (diff):
    DR = sudoku.DifficultyRating
    if diff < DR.easy_range[1]:
        if diff < DR.easy_range[1] / 3:
            c = '#8ae234' # green
        elif diff < 2 * DR.easy_range[1] / 3:
            c = '#73d216'
        else:
            c = '#4e9a06'
    elif diff < DR.medium_range[1]:
        span = DR.medium_range[1] - DR.easy_range[1]
        if diff < DR.medium_range[0] + (span / 3):
            c = '#204a87' # sky blue
        elif diff < DR.medium_range[0] + (2 * (span / 3)):
            c = '#3465a4'
        else:
            c = '#729fcf'
    elif diff < DR.hard_range[1]:
        span = DR.hard_range[1] - DR.medium_range[1]
        if diff < DR.hard_range[0] + span / 3:
            c = '#fcaf3e' # orange
        elif diff < DR.hard_range[0] + span * 2 / 3:
            c = '#f57900'
        else:
            c = '#ce5c00'
    else:
        span = DR.very_hard_range[1] - DR.hard_range[1]
        if diff < DR.very_hard_range[0] + span / 3:
            c = '#ef2929' # scarlet red
        elif diff < DR.very_hard_range[0]+ span * 2 / 3:
            c = '#cc0000'
        else:
            c = '#a40000'
    return color_hex_to_float(c)

class NewOrSavedGameSelector:

    NEW_GAME = 0
    SAVED_GAME = 1

    ui_file = os.path.join(defaults.UI_DIR, 'select_game.ui')

    @simple_debug
    def __init__ (self):
        self.sudoku_maker = sudoku_maker.SudokuMaker()
        self.dialog = None
        self.puzzle = None
        self.new_game_model = None
        self.saved_game_model = None
        self.saved_games = None

    def setup_dialog (self):
        builder = Gtk.Builder()
        builder.set_translation_domain(defaults.DOMAIN)
        builder.add_from_file(self.ui_file)
        self.dialog = builder.get_object('dialog1')
        self.dialog.set_default_response(Gtk.ResponseType.CANCEL)
        self.dialog.connect('close', self.close)
        self.dialog.hide()
        saved_game_frame = builder.get_object('savedGameFrame')
        saved_game_view = builder.get_object('savedGameIconView')
        saved_game_widgets = [
            saved_game_frame,
            saved_game_view,
            builder.get_object('savedGameLabel')
            ]
        builder.get_object('savedGameLabel').set_mnemonic_widget(
            saved_game_view
            )
        new_game_frame = builder.get_object('newGameFrame')
        new_game_frame.show()
        new_game_view = builder.get_object('newGameIconView')
        builder.get_object('newGameLabel').set_mnemonic_widget(
            new_game_view
            )
        self.saved_games = saver.SudokuTracker().list_saved_games()
        self.make_new_game_model()
        new_game_view.set_model(self.new_game_model)
        new_game_view.set_markup_column(0)
        new_game_view.set_pixbuf_column(1)
        self.make_saved_game_model()
        if len(self.saved_game_model) == 0:
            for w in saved_game_widgets:
                w.hide()
        else:
            saved_game_frame.show()
            self.saved_game_model.set_sort_column_id(2, Gtk.SortType.DESCENDING)
            saved_game_view.set_model(self.saved_game_model)
            saved_game_view.set_markup_column(0)
            saved_game_view.set_pixbuf_column(1)
        for view in saved_game_view, new_game_view:
            view.set_item_width(150)
            view.set_columns(4)
            view.set_spacing(12)
            view.set_selection_mode(Gtk.SelectionMode.SINGLE)
        saved_game_view.connect('item-activated', self.saved_item_activated_cb)
        saved_game_view.connect('button-release-event', self.item_clicked_cb)
        new_game_view.connect('item-activated', self.new_item_activated_cb)
        new_game_view.connect('button-release-event', self.item_clicked_cb)

    @simple_debug
    def make_new_game_model (self):
        # Description, Pixbuf, Puzzle (str)
        self.new_game_model = Gtk.ListStore(str, GdkPixbuf.Pixbuf, str)
        saved_games_to_exclude = [
            g['game'].split('\n')[0] for g in self.saved_games
            ]
        for cat in sudoku.DifficultyRating.ordered_categories:
            label = {'easy': _('Easy'),
                     'medium': _('Medium'),
                     'hard': _('Hard'),
                     'very hard': _('Very hard')}[cat]
            puzzles = self.sudoku_maker.get_puzzles(1, [cat], new = True,
                                                    exclude = saved_games_to_exclude
                                                    )
            if puzzles:
                puzzle, diff_val = puzzles[0]
            else:
                print 'WARNING: Repeating puzzle for difficulty %s -- generate more puzzles to avoid this.' % cat
                puzzles = self.sudoku_maker.get_puzzles(1, [cat], new = False)
                if puzzles:
                    puzzle, diff_val = puzzles[0]
                    lpuz = list(puzzle)
                    lpuz.reverse() # we reverse the puzzle so it at least looks different :-)
                    puzzle = ''
                    for n in lpuz:
                        puzzle += n
                else:
                    print 'WARNING: No puzzle for difficulty', cat
                    continue
            grid = sudoku.sudoku_grid_from_string(puzzle).grid
            self.new_game_model.append(('<b><i>' + label + '</i></b>',
                                        sudoku_thumber.make_pixbuf(grid,
                                                                   None,
                                                                   color_from_difficulty(diff_val)
                                                                   ),
                                        puzzle
                                        ))

    def format_friendly_date (self, tim):
        local_tim = time.localtime(tim)
        diff = int(time.time() - tim)
        curr_hour, curr_min = time.localtime()[3:5]
        now_to_yesterday = curr_hour * 60 * 60 + curr_min * 60
        if diff < now_to_yesterday:
            # Then we're today
            if diff < 60: # within the minute
                return ngettext("Last played %(n)s second ago",
                                "Last played %(n)s seconds ago", diff) % {'n': diff}
            elif diff < (60 * 60): # within the hour...
                minute = int(diff / 60)
                return ngettext("Last played %(n)s minute ago",
                                "Last played %(n)s minutes ago", minute) % {'n': minute}
            else:
                # Translators, see strftime manual in order to translate %? format strings
                return time.strftime(_("Last played at %I:%M %p"), local_tim)
        elif diff < now_to_yesterday + (60 * 60 * 24):
            # Translators, see strftime manual in order to translate %? format strings
            return time.strftime(_("Last played yesterday at %I:%M %p"), local_tim)
        elif diff < now_to_yesterday + (60 * 60 * 24) * 6:
            # Translators, see strftime manual in order to translate %? format strings
            return time.strftime(_("Last played on %A at %I:%M %p"), local_tim)
        else:
            # Translators, see strftime manual in order to translate %? format strings
            return time.strftime(_("Last played on %B %e %Y"), local_tim)

    @simple_debug
    def make_saved_game_model (self):
        # Description, Image, Last-Access time (for sorting), Puzzle (jar)
        self.saved_game_model = Gtk.ListStore(str, GdkPixbuf.Pixbuf, int, GObject.TYPE_PYOBJECT)
        for g in self.saved_games:
            game = g['game'].split('\n')[0]
            grid = sudoku.sudoku_grid_from_string(game)
            sr = sudoku.SudokuRater(grid.grid)
            sdifficulty = sr.difficulty()
            lastPlayedText = self.format_friendly_date(g['saved_at'])
            try:
                levelText = {'easy': _('Easy puzzle'),
                             'medium': _('Medium puzzle'),
                             'hard': _('Hard puzzle'),
                             'very hard': _('Very hard puzzle')}[sdifficulty.value_category()]
            except KeyError:
                levelText = sdifficulty.value_category()
            tim = g['timer.active_time']
            if tim >= 3600.0:
                hours = int(tim / 3600)
                durationText = ngettext("Played for %d hour", "Played for %d hours", hours) % hours
            elif tim >= 60.0:
                minutes = int(tim / 60)
                durationText = ngettext("Played for %d minute", "Played for %d minutes", minutes) % minutes
            else:
                seconds = int(tim)
                durationText = ngettext("Played for %d second", "Played for %d seconds", seconds) % seconds            
            desc = "<b><i>%s</i></b>\n<span size='small'><i>%s</i>\n<i>%s.</i></span>" % (
                levelText,
                lastPlayedText,
                durationText,
                )
            self.saved_game_model.append((
                desc,
                sudoku_thumber.make_pixbuf(grid.grid,
                                           sudoku.sudoku_grid_from_string(g['game'].split('\n')[1].replace(' ', '')).grid,
                                           color_from_difficulty(sdifficulty.value)
                                           ),
                g['saved_at'],
                g
                ))

    @simple_debug
    def new_item_activated_cb (self, iconview, path):
        self.play_game(iconview.get_model()[path][2])

    @simple_debug
    def saved_item_activated_cb (self, iconview, path):
        self.resume_game(iconview.get_model()[path][3])

    @simple_debug
    def item_clicked_cb (self, wdgt, evt):
        '''Single click event handler

        This callback activates the selected icon with a single click.
        '''
        if evt.button != 1:
            return False
        # See if the player clicked an icon
        position = wdgt.get_path_at_pos(int(evt.x), int(evt.y))
        if not position:
            return False
        if not wdgt.path_is_selected(position):
            return False
        # Select it, redraw it(strictly aesthetic), and then activate it
        wdgt.select_path(position)
        wdgt.get_parent_window().process_updates(False)
        wdgt.item_activated(position)

    @simple_debug
    def resume_game (self, jar):
        self.puzzle = (self.SAVED_GAME, jar)
        self.dialog.emit('response', Gtk.ResponseType.OK)

    @simple_debug
    def play_game (self, puzzle):
        self.puzzle = (self.NEW_GAME, puzzle)
        self.dialog.emit('response', Gtk.ResponseType.OK)

    @simple_debug
    def close (self):
        self.dialog.emit('response', Gtk.ResponseType.CLOSE)

    @simple_debug
    def handle_response (self, response):
        if response == Gtk.ResponseType.OK:
            return self.puzzle
        else:
            return None

    def run_swallowed_dialog (self, swallower):
        self.setup_dialog()
        return self.handle_response(
            swallower.run_dialog(self.dialog)
            )

    def run_dialog (self):
        self.setup_dialog()
        return self.handle_response(self.dialog.run())

if __name__ == '__main__':
    selector = NewOrSavedGameSelector()
    import pprint
    pprint.pprint (selector.run_dialog())



# -*- coding: utf-8 -*-
import os, shutil
import errno
import sudoku
import random
import time
import pausable
import threading
import defaults

class SudokuGenerator:

    """A class to generate new Sudoku Puzzles."""

    def __init__ (self, start_grid = None, clues = 2, group_size = 9):
        self.generated = []
        self.clues = clues
        self.all_coords = []
        self.group_size = group_size
        for x in range(self.group_size):
            for y in range(self.group_size):
                self.all_coords.append((x, y))
        if start_grid:
            self.start_grid = sudoku.SudokuGrid(start_grid)
        else:
            try:
                self.start_grid = self.generate_grid()
            except:
                self.start_grid = self.generate_grid()
        self.puzzles = []
        self.rated_puzzles = []

    def average_difficulty (self):
        difficulties = [i[1].value for i in self.rated_puzzles]
        if difficulties:
            return sum(difficulties)/len(difficulties)

    def generate_grid (self):
        self.start_grid = sudoku.SudokuSolver(verbose = False, group_size = self.group_size)
        self.start_grid.solve()
        return self.start_grid

    def reflect (self, x, y, axis = 1):
        #downward sloping
        upper = self.group_size - 1
        # reflect once...
        x, y = upper - y, upper-x
        # reflect twice...
        return y, x

    def make_symmetric_puzzle (self):
        nclues = self.clues/2
        buckshot = set(random.sample(self.all_coords, nclues))
        new_puzzle = sudoku.SudokuGrid(verbose = False, group_size = self.group_size)
        reflections = set()
        for x, y in buckshot:
            reflection = self.reflect(x, y)
            if reflection:
                nclues += 1
                reflections.add(reflection)
        buckshot = buckshot | reflections # unite our sets
        remaining_coords = set(self.all_coords) - set(buckshot)
        while len(buckshot) < self.clues:
            coord = random.sample(remaining_coords, 1)[0]
            buckshot.add(coord)
            reflection = self.reflect(*coord)
            if reflection:
                buckshot.add(reflection)
            remaining_coords = remaining_coords - buckshot
        return self.make_puzzle_from_coords(buckshot)

    def make_puzzle (self):
        buckshot = random.sample(self.all_coords, self.clues)
        while buckshot in self.generated:
            buckshot = random.sample(self.all_coords, self.clues)
        return self.make_puzzle_from_coords(buckshot)

    def make_puzzle_from_coords (self, buckshot):
        new_puzzle = sudoku.SudokuGrid(verbose = False, group_size = self.group_size)
        self.generated.append(set(buckshot))
        for x, y in buckshot:
            new_puzzle.add(x, y, self.start_grid._get_(x, y))
        self.puzzles.append(new_puzzle)
        return new_puzzle

    def make_puzzle_by_boxes (self,
                              skew_by = 0.0,
                              max_squares = None):
        """Make a puzzle paying attention to evenness of clue
        distribution.

        If skew_by is 0, we distribute our clues as evenly as possible
        across boxes.  If skew by is 1.0, we make the distribution of
        clues as uneven as possible. In other words, if we had 27
        boxes for a 9x9 grid, a skew_by of 0 would put exactly 3 clues
        in each 3x3 grid whereas a skew_by of 1.0 would completely
        fill 3 3x3 grids with clues.

        We believe this skewing may have something to do with how
        difficult a puzzle is to solve. By toying with the ratios,
        this method may make it considerably easier to generate
        difficult or easy puzzles.
        """
        # Number of total boxes
        nboxes = len(self.start_grid.boxes)
        # If no max is given, we calculate one based on our skew_by --
        # a skew_by of 1 will always produce full squares, 0 will
        # produce the minimum fullness, and between between in
        # proportion to its betweenness.
        if not max_squares:
            max_squares = self.clues / nboxes
            max_squares += int((nboxes-max_squares)*skew_by)
        clued = 0
        # nclues will be a list of the number of clues we want per
        # box.
        nclues = []
        for n in range(nboxes):
            # Make sure we'll have enough clues to fill our target
            # number, regardless of our calculation of the current max
            minimum = (self.clues-clued)/(nboxes-n)
            if max_squares < minimum:
                cls = minimum
            else:
                cls = int(max_squares)
            clues = max_squares
            if clues > (self.clues - clued):
                clues = self.clues - clued
            nclues.append(int(clues))
            clued += clues
            if skew_by:
                # Reduce our number of squares proportionally to
                # skewiness.
                max_squares = round(max_squares * skew_by)
        # shuffle ourselves...
        random.shuffle(nclues)
        buckshot = []
        for i in range(nboxes):
            if nclues[i]:
                buckshot.extend(
                    random.sample(self.start_grid.box_coords[i],
                                  nclues[i])
                    )
        return self.make_puzzle_from_coords(buckshot)

    def assess_difficulty (self, sudoku_grid):
        try:
            solver = sudoku.SudokuRater(sudoku_grid, verbose = False, group_size = self.group_size)
            d = solver.difficulty()
            self.rated_puzzles.append((sudoku_grid, d))
            return d
        except:
            print 'Impossible!'
            print 'Puzzle was:'
            print solver.virgin
            print 'Solution: ',
            print self.start_grid
            print 'Puzzle foobared in following state:',
            print solver
            raise

    def is_unique (self, sudoku_grid):
        """If puzzle is unique, return its difficulty.

        Otherwise, return None."""
        solver = sudoku.SudokuRater(sudoku_grid, verbose = False, group_size = self.group_size)
        if solver.has_unique_solution():
            return solver.difficulty()
        else:
            return None

    def generate_puzzle_for_difficulty (self,
                                        lower_target = 0.3,
                                        upper_target = 0.5,
                                        max_tries = 100,
                                        by_box = False,
                                        by_box_kwargs = {}):
        for i in range(max_tries):
            if by_box:
                puz = self.make_puzzle_by_boxes(**by_box_kwargs)
            else:
                puz = self.make_puzzle()
            d = self.assess_difficulty(puz.grid)
            if (d and (not lower_target or d.value > lower_target) and\
               (not upper_target or
                d.value < upper_target)):
                return puz, d
        else:
            return None, None

    def make_unique_puzzle (self, symmetrical = True, strict_number_of_clues = False):
        if symmetrical:
            puz = self.make_symmetric_puzzle()
        else:
            puz = self.make_puzzle()
        diff = False
        if self.clues > 10:
            clues = self.clues - 8
        else:
            clues = 2
        while 1:
            solver = sudoku.SudokuRater(puz.grid, verbose = False, group_size = self.group_size)
            if solver.has_unique_solution():
                diff = solver.difficulty()
                #raw_input('Unique puzzle!')
                break
            # Otherwise...
            crumb = solver.breadcrumbs[-1]
            fill_in = [(crumb.x, crumb.y)]
            reflection = self.reflect(crumb.x, crumb.y)
            if reflection:
                fill_in.append(reflection)
            for x, y in fill_in:
                solver.virgin._set_(x, y, self.start_grid._get_(x, y))
            puz = sudoku.SudokuGrid(solver.virgin.grid, verbose = False)
            #print 'Not unique, adding ',fill_in
            clues += len(fill_in)
            if strict_number_of_clues  == True and clues > self.clues:
                return None
            #print clues, "clues..."
            #raw_input('Continue: ')
        # make sure we have the proper number of clues
        if strict_number_of_clues:
            changed = False
            while clues < self.clues:
                x, y = random.randint(0, 8), random.randint(0, 8)
                while puz._get_(x, y):
                    x, y = random.randint(0, 8), random.randint(0, 8)
                puz._set_(x, y, self.start_grid._get_(x, y))
                clues += 1
                reflection = self.reflect(x, y)
                if reflection:
                    puz._set_(x, y, self.start_grid._get_(x, y))
                    clues += 1
                changed = True
            if changed:
                diff = sudoku.SudokuRater(puz.grid,
                                                  verbose = False,
                                                  group_size = self.group_size).difficulty()
        return puz, diff

    def make_unique_puzzles (self, n = 10, ugargs = {}):
        ug = self.unique_generator(**ugargs)
        ret = []
        for i in range(n):
            #print 'Working on puzzle ',i
            ret.append(ug.next())
            #print 'Got one!'
        return ret

    def unique_generator (self,
                          symmetrical = True,
                          strict_number_of_clues = False,
                          by_box = False,
                          by_box_kwargs = {}):
        while 1:
            result = self.make_unique_puzzle(
                symmetrical = symmetrical,
                strict_number_of_clues = strict_number_of_clues)
            if result:
                yield result

    def generate_puzzles (self, n = 10,
                          symmetrical = True,
                          by_box = False,
                          by_box_kwargs = {}):
        ret = []
        for i in range(n):
            #print 'Generating puzzle ',i
            if symmetrical:
                puz = self.make_symmetric_puzzle()
            elif by_box:
                puz = self.make_puzzle_by_boxes(**by_box_kwargs)
            else:
                puz = self.make_puzzle()
            #print 'Assessing puzzle ',puz
            try:
                d = self.assess_difficulty(puz.grid)
            except:
                raise
            if d:
                ret.append((puz, d))
        ret.sort(lambda a, b: a[1].value>b[1].value and 1 or a[1].value<b[1].value and -1 or 0)
        return ret


class InterruptibleSudokuGenerator (SudokuGenerator):
    def __init__ (self, *args, **kwargs):
        self.paused = False
        self.terminated = False
        SudokuGenerator.__init__(self, *args, **kwargs)
    def work (self, *args, **kwargs):
        self.unterminate()
        SudokuGenerator(self, *args, **kwargs)

pausable.make_pausable(InterruptibleSudokuGenerator)


class SudokuMaker:

    """A class to create unique, symmetrical sudoku puzzles."""

    def __init__ (self,
                  generator_args = {'clues':27,
                                  'group_size':9},
                  puzzle_maker_args = {'symmetrical':True},
                  batch_size = 5,
                  pickle_to = os.path.join(defaults.DATA_DIR, 'puzzles')):
        self.pickle_to = pickle_to
        self.paused = False
        self.terminated = False
        self.generator_args = generator_args
        self.puzzle_maker_args = puzzle_maker_args
        self.batch_size = batch_size
        self.load()
        self.all_puzzles = {}
        self.played = self.get_pregenerated('finished')
        self.n_available_sudokus = {'easy':None, 'medium':None, 'hard':None, 'very hard':None}

    def load (self):
        try:
            os.makedirs(self.pickle_to)
        except os.error, e:
            if e.errno != errno.EEXIST:
                return
        for cat in sudoku.DifficultyRating.categories:
            source = os.path.join(os.path.join(defaults.PUZZLE_DIR), cat.replace(' ', '_'))
            target = os.path.join(self.pickle_to, cat.replace(' ', '_'))
            if not os.path.exists(target):
                try:
                    shutil.copy(source, target)
                except:
                    print 'Problem copying base puzzles'
                    print 'Attempted to copy from %s to %s' % (source, target)

    def get_pregenerated (self, difficulty):
        fname = os.path.join(self.pickle_to, difficulty.replace(' ', '_'))
        try:
            lines = file(fname).readlines()
        except IOError, e:
            if e.errno != errno.ENOENT:
                print 'Error reading pregenerated puzzles for difficulty \'%s\': %s' % (difficulty, e.strerror)
            return []
        else:
            return [line.strip() for line in lines]

    def n_puzzles (self, difficulty_category = None, new = True):
        if not difficulty_category:
            return sum([self.n_puzzles(c, new = new) for c in sudoku.DifficultyRating.categories])
        else:
            if self.n_available_sudokus[difficulty_category]:
                return self.n_available_sudokus[difficulty_category]
            lines = self.get_pregenerated(difficulty_category)
            count = 0
            for line in lines:
                if (not new) or line.split('\t')[0] not in self.played:
                    count += 1
            self.n_available_sudokus[difficulty_category] = count
            return self.n_available_sudokus[difficulty_category]

    def list_puzzles (self, difficulty_category = None, new = True):
        """Return a list of all puzzles we have generated.
        """
        puzzle_list = []
        if not difficulty_category:
            for c in sudoku.DifficultyRating.categories:
                puzzle_list.extend(self.list_puzzles(c, new = new))
        else:
            lines = self.get_pregenerated(difficulty_category)
            for l in lines:
                puzzle = l.split('\t')[0]
                if (not new) or puzzle not in self.played:
                    puzzle_list.append(puzzle)
        return puzzle_list

    def get_puzzles_random (self, n, levels, new = True, exclude = []):
        """Return a list of n puzzles and difficulty values (as floats).

        The puzzles will correspond as closely as possible to levels.
        If new, we only return puzzles not yet played.
        """
        if not n:
            return []
        assert(levels)
        puzzles = []
        # Open files to read puzzles...
        puzzles_by_level = {}
        files = {}
        for l in levels:
            puzzles_by_level[l] = self.get_pregenerated(l)
            random.shuffle(puzzles_by_level[l])
        i = 0
        il = 0
        n_per_level = {}
        finished = []
        while i < n and len(finished) < len(levels):
            if il >= len(levels):
                il = 0
            lev = levels[il]
            # skip any levels that we've exhausted
            if lev in finished:
                il += 1
                continue
            try:
                line = puzzles_by_level[lev].pop()
            except IndexError:
                finished.append(lev)
            else:
                try:
                    p, d = line.split('\t')
                except ValueError:
                    print 'WARNING: invalid line %s in file %s' % (line, files[lev])
                    continue
                if sudoku.is_valid_puzzle(p):
                    if (p not in exclude) and (not new or p not in self.played):
                        puzzles.append((p, float(d)))
                        i += 1
                else:
                    print 'WARNING: invalid puzzle %s in file %s' % (p, files[lev])
            il += 1
        if i < n:
            print 'WARNING: Not able to provide %s puzzles in levels %s' % (n, levels)
            print 'WARNING: Generate more puzzles if you really need this many puzzles!'
        return puzzles

    def get_puzzles (self, n, levels, new = True, randomize = True,
                     exclude = []):
        """Return a list of n puzzles and difficulty values (as floats).

        The puzzles will correspond as closely as possible to levels.
        If new, we only return puzzles not yet played.
        """
        if randomize:
            return self.get_puzzles_random(n, levels, new = new, exclude = exclude)
        if not n:
            return []
        assert(levels)
        puzzles = []

        # Open files to read puzzles...
        files = {}
        for l in levels:
            files[l] = self.get_pregenerated(l)

        i = 0
        il = 0
        n_per_level = {}
        finished = []
        while i < n and len(finished) < len(levels):
            if il >= len(levels):
                il = 0
            lev = levels[il]
            # skip any levels that we've exhausted
            if lev in finished:
                il += 1
                continue

            if len(files[lev]) == 0:
                finished.append(lev)
            else:
                line = files[lev][0]
                files[lev] = files[lev][1:]
                try:
                    p, d = line.split('\t')
                except ValueError:
                    print 'WARNING: invalid line %s in file %s' % (line, files[lev])
                    continue
                if sudoku.is_valid_puzzle(p):
                    if (p not in exclude) and (not new or p not in self.played):
                        puzzles.append((p, float(d)))
                        i += 1
                else:
                    print 'WARNING: invalid puzzle %s in file %s' % (p, files[lev])
            il += 1
        if i < n:
            print 'WARNING: Not able to provide %s puzzles in levels %s' % (n, levels)
            print 'WARNING: Generate more puzzles if you really need this many puzzles!'

        return puzzles

    # End convenience methods for accessing puzzles we've created

    # Methods for creating new puzzles

    def make_batch (self, diffs = None):
        self.new_generator = InterruptibleSudokuGenerator(**self.generator_args)
        key = self.new_generator.start_grid.to_string()
        #while
        #    self.new_generator = InterruptibleSudokuGenerator(**self.generator_args)
        #    key = self.new_generator.start_grid.to_string()
        #print 'We have our solution grid'
        #self.puzzles_by_solution[key]=[]
        ug = self.new_generator.unique_generator(**self.puzzle_maker_args)
        open_files = {}
        for n in range(self.batch_size):
            #print 'start next item...',n
            puz, diff = ug.next()
            #print "GENERATED ",puz,diff
            if (diffs == None or diff.value_category() in diffs):
                puzstring = puz.to_string()
                # self.puzzles_by_solution[key].append((puzstring,diff))
                # self.solutions_by_puzzle[puzstring]=key
                # self.all_puzzles[puzstring] = diff
                # self.names[puzstring] = self.get_puzzle_name(_('Puzzle'))
                outpath = os.path.join(self.pickle_to,
                                       diff.value_category().replace(' ', '_'))
                # Read through the existing file and make sure we're
                # not a duplicate puzzle
                existing = self.get_pregenerated(diff.value_category())
                if not puzstring in existing:
                    try:
                        outfi = file(outpath, 'a')
                        outfi.write(puzstring+'\t'+str(diff.value)+'\n')
                        outfi.close()
                        self.n_available_sudokus[diff.value_category()]+=1
                    except IOError, e:
                        print 'Error appending pregenerated puzzle: %s' % e.strerror

    def pause (self, *args):
        if hasattr(self, 'new_generator'):
            self.new_generator.pause()
        self.paused = True

    def resume (self, *args):
        if hasattr(self, 'new_generator'):
            self.new_generator.resume()
        self.paused = False

    def stop (self, *args):
        if hasattr(self, 'new_generator'):
            self.new_generator.terminate()
        self.terminated = True

    def hesitate (self):
        while self.paused:
            if self.terminated:
                break
            time.sleep(1)

    def work (self, limit = None, diffs = None):
        """Intended to be called as a worker thread, make puzzles!"""
        self.terminated = False
        if hasattr(self, 'new_generator'):
            self.new_generator.termintaed = False
        self.paused = False
        generated = 0
        while not limit or generated < limit:
            if self.terminated:
                break
            if self.paused:
                self.hesitate()
            try:
                self.make_batch(diffs = diffs)
            except:
                raise
            else:
                generated += 1

    def get_difficulty (self, puz):
        return sudoku.SudokuRater(puz).difficulty()


if __name__ == '__main__':
    #puzzles=sg.generate_puzzles(30)
    #
    #print sg.make_symmetric_puzzle()
    #unique_maker = sg.unique_generator()
    #unique = []
    #for n in range(10):
    #    unique.append(unique_maker.next())
    #    print 'Generated Unique...'
    #unique_puzzles=filter(lambda x: sudoku.SudokuSolver(x[0].grid,verbose=False).has_unique_solution(),puzzles)
    sm = SudokuMaker()
    #st = SudokuTracker(sm)
elif False:
    usage = """Commands are:
    run: Run sudoku-maker
    len: \# of puzzles generated
    pause: pause
    resume: resume
    terminate: kill thread
    quit: to quit
    """
    print usage
    while 1:
        inp = raw_input('choose:')
        if inp == 'run':
            t = threading.Thread(target = sm.make_batch)
            t.start()
        elif inp == 'show':
            print sm.puzzles
        elif inp == 'len':
            print len(sm.puzzles)
        elif inp == 'pause':
            sm.new_generator.pause()
        elif inp == 'resume':
            sm.new_generator.resume()
        elif inp == 'terminate':
            sm.new_generator.terminate()
        elif inp == 'quit':
            sm.new_generator.terminate()
            break
        else:
            try:
                getattr(sm, inp)()
            except:
                print usage
        



# -*- coding: utf-8 -*-
#!/usr/bin/python

import random
import copy

NO_TRACKER = -1 # Tracker id for untracked values

class TrackerInfo(object):
    '''Tracker state machine(singleton)

    The singleton instance of this class is used to manipulate tracker
    selection and tracked values, as well as interrogate tracker colors.

    _tracks - dictionary for tracked values.  The tracker id is used as the
        key.  A tracker is a dictionary that stored tracked values keyed by
        its coordinates(x, y).  _tracks[tracker_id][(x, y)] == tracked value

    current_tracker - The tracker id for the currently selected tracker
    showing_tracker - The tracker id for the tracker that is currently being
        viewed.  The point to this member is to store off the tracker when
        the player switches to "Untracked" so that the last tracker they were
        working on stays in view after the switch.
    '''
    __single = None
    _tracks = {}
    _colors = {}
    current_tracker = NO_TRACKER
    showing_tracker = NO_TRACKER

    def __new__(cls, *args, **kwargs):
        '''Overridden to implement as a singleton
        '''
        # Check to see if a __single exists already for this class
        # Compare class types instead of just looking for None so
        # that subclasses will create their own __single objects
        if cls != type(cls.__single):
            cls.__single = object.__new__(cls, *args, **kwargs)
        return cls.__single

    def __init__(self):
        # Only initialize the colors once
        if self._colors:
            return
        # Use tango colors recommended here:
        # http://tango.freedesktop.org/Tango_Icon_Theme_Guidelines
        for tracker_id, cols in enumerate(
        [(32, 74, 135), # Sky Blue 3
         (78, 154, 6), # Chameleon 3
         (206, 92, 0), # Orange 3
         (143, 89, 2), # Chocolate 3
         (92, 53, 102), # Plum 3
         (85, 87, 83), # Aluminium 5
         (196, 160, 0) # Butter 3
        ]):
            self._colors[tracker_id] = tuple([x / 255.0 for x in cols])

    def load(self, pickle):
        self.current_tracker, self.showing_tracker, self._tracks = pickle

    def save(self):
        return (self.current_tracker, self.showing_tracker, self.get_trackers())

    def create_tracker (self, tracker_id = 0):
        '''Create storage for a new tracker

        tracker_id can be passed in to attempt creation of a specific id, but
        if the tracker_id already exists then the passed number will be
        incremented until a suitable key can be allocated.
        '''
        if not tracker_id:
            tracker_id = 0
        while self._tracks.has_key(tracker_id):
            tracker_id += 1
        self._tracks[tracker_id] = {}
        return tracker_id

    def get_tracker(self, tracker_id):
        if self._tracks.has_key(tracker_id):
            return self._tracks[tracker_id]

    def delete_tracker(self, tracker_id):
        if self._tracks.has_key(tracker_id):
            del self._tracks[tracker_id]

    def reset (self):
        ''' Reset the tracker information
        '''
        self._tracks = {}
        self.current_tracker = NO_TRACKER
        self.showing_tracker = NO_TRACKER

    def use_trackers (self, trackers):
        self._tracks = trackers

    def get_trackers(self):
        return copy.deepcopy(self._tracks)

    def set_tracker(self, tracker_id):
        self.current_tracker = tracker_id
        if tracker_id != NO_TRACKER:
            self.showing_tracker = tracker_id

    def hide_tracker(self):
        self.showing_tracker = NO_TRACKER

    def get_tracker_view(self):
        return((self.current_tracker, self.showing_tracker))

    def set_tracker_view(self, tview):
        self.current_tracker, self.showing_tracker = tview

    def get_color (self, tracker_id):
        # Untracked items don't get specially colored
        if tracker_id == NO_TRACKER:
            return None
        # Create a random color for new trackers that are beyond the defaults
        if not self._colors.has_key(tracker_id):
            random_color = self._colors[0]
            while random_color in self._colors.values():
                # If we have generated all possible colors, this will
                # enter an infinite loop
                random_color = (random.randint(0, 100)/100.0,
                                random.randint(0, 100)/100.0,
                                random.randint(0, 100)/100.0)
            self._colors[tracker_id] = random_color
        return self._colors[tracker_id]

    def get_color_markup(self, tracker_id):
        color_tuple = self.get_color (tracker_id)
        if not color_tuple:
            return None
        color_markup = '#'
        color_markup += str(hex(int(color_tuple[0]*255))[2:]).zfill(2)
        color_markup += str(hex(int(color_tuple[1]*255))[2:]).zfill(2)
        color_markup += str(hex(int(color_tuple[2]*255))[2:]).zfill(2)
        return color_markup.upper()

    def get_current_color(self):
        return self.get_color(self.current_tracker)

    def get_showing_color(self):
        return self.get_color(self.showing_tracker)

    def add_trace(self, x, y, value, tracker_id = None):
        '''Add a tracked value

        By default(tracker_id set to None) this method adds a value to the
        current tracker.  tracker_id can be passed in to add it to a specific
        tracker.
        '''
        if tracker_id == None:
            to_tracker = self.current_tracker
        else:
            to_tracker =  tracker_id
        # Need a tracker
        if to_tracker == NO_TRACKER:
            return
        # Make sure the dictionary is available for the tracker.
        if not self._tracks.has_key(to_tracker):
            self._tracks[to_tracker] = {}
        # Add it
        self._tracks[to_tracker][(x, y)] = value

    def remove_trace(self, x, y, from_tracker = None):
        '''Remove a tracked value

        By default(from_tracker set to None) this method removes all tracked
        values for a particular cell(x, y coords).  from_tracker can be passed
        to remove tracked values from a particular tracker only.
        '''
        if from_tracker == None:
            from_tracks = self._tracks.keys()
        else:
            from_tracks = [from_tracker]
        # Delete them
        for tracker in from_tracks:
            if self._tracks.has_key(tracker) and self._tracks[tracker].has_key((x, y)):
                del self._tracks[tracker][(x, y)]

    def get_trackers_for_cell(self, x, y):
        '''Return all trackers for a cell

        This function is used for the undo mechanism.  A list in the format
        (tracker, value) is returned so that it may later be reinstated with
        reset_trackers_for_cell().
        '''
        ret = []
        for tracker, track in self._tracks.items():
            if track.has_key((x, y)):
                ret.append((tracker, track[(x, y)]))
        return ret

    def reset_trackers_for_cell(self, x, y, old_trackers):
        '''Reset all trackers to a previous state for a cell

        This function is used for the undo mechanism.  It reinstates the
        tracked values the list created by get_trackers_for_cell().
        '''
        # Remove all the current traces
        for tracker, track in self._tracks.items():
            if track.has_key((x, y)):
                del self._tracks[tracker][(x, y)]
        # Add the old ones back
        for tracker, value in old_trackers:
            self._tracks[tracker][(x, y)] = value





# -*- coding: utf-8 -*-
import sys

# Ignore any exceptions writing to stdout using print statements
class SafeStdout:
    def __init__(self):
        self.stdout = sys.stdout

    def fileno(self):
        return self.stdout.fileno()

    def write(self, data):
        try:
            self.stdout.write(data)
        except:
            pass

sys.stdout = SafeStdout()

def start_game ():
    import main
    main.start_game()



# -*- coding: utf-8 -*-
import os.path
import threading

import gi
gi.require_version("Gtk", "3.0")

from gi.repository import Gtk,GdkPixbuf,GObject,Pango,Gdk,Gio
from gettext import gettext as _
from gettext import ngettext

import cairo
import dialog_swallower
import game_selector
import gsudoku
import printing
import saver
import sudoku_maker
import timer
import tracker_info
from defaults import (APPNAME, APPNAME_SHORT, AUTHORS, COPYRIGHT, DESCRIPTION, DOMAIN, 
        IMAGE_DIR, MIN_NEW_PUZZLES, UI_DIR, VERSION, WEBSITE, WEBSITE_LABEL)
from gtk_goodies import Undo, dialog_extras
from simple_debug import simple_debug, options

def inactivate_new_game_etc (fun):
    def inactivate_new_game_etc_ (ui, *args, **kwargs):
        paths = [
            '/MenuBar/Game/New',
            '/MenuBar/Game/Reset',
            '/MenuBar/Game/PuzzleInfo',
            '/MenuBar/Game/Print',
            # undo/redo is handled elsewhere as it can't simply be turned on/off.
            '/MenuBar/Settings/ToggleToolbar',
            '/MenuBar/Settings/ToggleHighlight',
            '/MenuBar/Settings/AlwaysShowPossible',
            '/MenuBar/Settings/ShowImpossibleImplications',
            '/MenuBar/Tools/ShowPossible',
            '/MenuBar/Tools/ClearTopNotes',
            '/MenuBar/Tools/ClearBottomNotes',
            '/MenuBar/Tools/Tracker',
            ]
        for p in paths:
            action = ui.uimanager.get_action(p)
            if not action:
                action = ui.uimanager.get_widget(p)
            if not action:
                print 'No action at path', p
            else:
                action.set_sensitive(False)
        ret = fun(ui, *args, **kwargs)
        for p in paths:
            action = ui.uimanager.get_action(p)
            if not action:
                action = ui.uimanager.get_widget(p)
            if not action:
                print 'No action at path', p
            else:
                action.set_sensitive(True)
        return ret
    return inactivate_new_game_etc_

class UI:
    ui = '''<ui>
    <menubar name="MenuBar">
      <menu name="Game" action="Game">
        <menuitem action="New"/>
        <menuitem action="Reset"/>
        <separator/>
        <menuitem action="Undo"/>
        <menuitem action="Redo"/>
        <separator/>
        <menuitem action="PuzzleInfo"/>
        <separator/>
        <menuitem action="Print"/>
        <menuitem action="PrintMany"/>
        <separator/>
        <menuitem action="Close"/>
      </menu>
      <menu action="Settings">
        <menuitem action="FullScreen"/>
        <menuitem action="ToggleToolbar"/>
        <separator/>
        <menuitem action="ToggleHighlight"/>
        <menuitem action="AlwaysShowPossible"/>
        <menuitem action="ShowImpossibleImplications"/>
      </menu>
      <menu action="Tools">
        <menuitem action="ShowPossible"/>
        <separator/>
        <menuitem action="ClearTopNotes"/>
        <menuitem action="ClearBottomNotes"/>
        <separator/>
        <menuitem action="Tracker"/>
      </menu>
      <menu action="Help">
        <menuitem action="ShowHelp"/>
        <menuitem action="About"/>
      </menu>
    </menubar>
    <toolbar name="Toolbar">
      <toolitem action="New"/>
      <separator/>
      <toolitem action="Undo"/>
      <toolitem action="Redo"/>
      <separator/>
      <toolitem action="ShowPossible"/>
      <separator/>
      <toolitem action="Tracker"/>
    </toolbar>
    </ui>'''

    @simple_debug
    def __init__ (self, run_selector = True):
        """run_selector means that we start our regular game.

        For testing purposes, it will be convenient to hand a
        run_selector=False to this method to avoid running the dialog
        and allow a tester to set up a game programmatically.
        """
        self.settings = Gio.Settings("org.gnome.gnome-sudoku")
        self.setup_gui()
        self.timer = timer.ActiveTimer(self.w)
        self.gsd.set_timer(self.timer)
        self.won = False
        # add the accelerator group to our toplevel window
        self.worker_connections = []
        self.is_fullscreen = False

        # setup sudoku maker...
        self.sudoku_maker = sudoku_maker.SudokuMaker()
        self.sudoku_tracker = saver.SudokuTracker()
        # generate puzzles while our use is working...
        self.show()
        if run_selector:
            self.do_stop()
            if self.select_game():
                # If this return True, the user closed...
                self.quit = True
            else:
                self.quit = False

        # Generate puzzles in background...
        GObject.timeout_add_seconds(1, lambda *args: self.start_worker_thread() and True)

    @inactivate_new_game_etc
    def select_game (self):
        self.tb.hide()
        choice = game_selector.NewOrSavedGameSelector().run_swallowed_dialog(self.swallower)
        if not choice:
            return True
        self.timer.start_timing()
        if choice[0] == game_selector.NewOrSavedGameSelector.NEW_GAME:
            self.gsd.change_grid(choice[1], 9)
        if choice[0] == game_selector.NewOrSavedGameSelector.SAVED_GAME:
            saver.open_game(self, choice[1])
        if self.settings.get_boolean('show-toolbar'):
            self.tb.show()
        if self.settings.get_boolean('always-show-hints'):
            self.gsd.update_all_hints()
        if self.settings.get_boolean('highlight'):
            self.gsd.toggle_highlight(True)


    def show (self):
        self.gsd.show()
        self.w.show()

    def setup_gui (self):
        self.setup_main_window()
        self.gsd = gsudoku.SudokuGameDisplay()
        self.gsd.set_parent_for(self.w)
        self.gsd.connect('puzzle-finished', self.you_win_callback)
        self.setup_color()
        self.setup_actions()
        self.setup_undo()
        self.setup_autosave()
        self.w.add_accel_group(self.uimanager.get_accel_group())
        self.setup_main_boxes()
        self.setup_tracker_interface()
        self.setup_toggles()

    def setup_main_window (self):
        Gtk.Window.set_default_icon_name('gnome-sudoku')
        self.w = Gtk.Window()
        self.w.set_default_size(self.settings.get_int('width'), self.settings.get_int('height'))
        self.w.set_title(APPNAME_SHORT)
        self.w.connect('configure-event', self.resize_cb)
        self.w.connect('delete-event', self.quit_cb)
        self.uimanager = Gtk.UIManager()

    def setup_actions (self):
        self.main_actions = Gtk.ActionGroup('MainActions')
        self.main_actions.add_actions([
            ('Game', None, _('_Game')),
            ('New', Gtk.STOCK_NEW, None, '<Control>n', _('New game'), self.new_cb),
            ('Reset', Gtk.STOCK_CLEAR, _('_Reset'), '<Control>b',
             None, self.game_reset_cb),
            ('Undo', Gtk.STOCK_UNDO, _('_Undo'), '<Control>z',
             _('Undo last action'), self.stop_dancer),
            ('Redo', Gtk.STOCK_REDO, _('_Redo'), '<Shift><Control>z',
             _('Redo last action')),
            ('PuzzleInfo', Gtk.STOCK_ABOUT, _('Puzzle _Statistics...'), None,
             None, self.show_info_cb),
            ('Print', Gtk.STOCK_PRINT, _('_Print...'), '<Control>p', None, self.print_game),
            ('PrintMany', Gtk.STOCK_PRINT, _('Print _Multiple Sudokus...'), None,
             None, self.print_multiple_games),
            ('Close', Gtk.STOCK_CLOSE, None, '<Control>w', None, self.quit_cb),
            ('Settings', None, _('_Settings')),
            ('FullScreen', Gtk.STOCK_FULLSCREEN, None, 'F11', None, self.full_screen_cb),
            ('Tools', None, _('_Tools')),
            ('ShowPossible', Gtk.STOCK_DIALOG_INFO, _('_Hint'), '<Control>h',
             _('Show a square that is easy to fill.'), self.show_hint_cb),
            ('ClearTopNotes', None, _('Clear _Top Notes'), '<Control>j',
             None, self.clear_top_notes_cb),
            ('ClearBottomNotes', None, _('Clear _Bottom Notes'), '<Control>k',
             None, self.clear_bottom_notes_cb),
            ('Help', None, _('_Help'), None, None, None),
            ('ShowHelp', Gtk.STOCK_HELP, _('_Contents'), 'F1', None, self.show_help),
            ('About', Gtk.STOCK_ABOUT, None, None, None, self.show_about),
            ])
        self.main_actions.add_toggle_actions([
            ('AlwaysShowPossible',
             None,
             _('Show _Possible Numbers'),
             None,
             _('Always show possible numbers in a square'),
             self.auto_hint_cb),
            ('ShowImpossibleImplications',
             None,
             _('Warn About _Unfillable Squares'),
             None,
             _('Warn about squares made unfillable by a move'),
             self.impossible_implication_cb),
            ('Tracker', 'tracks', _('_Track Additions'),
             '<Control>T',
             _('Mark new additions in a separate color so you can keep track of them.'),
             self.tracker_toggle_cb, False),
            ('ToggleToolbar', None, _('Show _Toolbar'), None, None, self.toggle_toolbar_cb, True),
            ('ToggleHighlight', Gtk.STOCK_SELECT_COLOR, _('_Highlighter'),
             None, _('Highlight the current row, column and box'), self.toggle_highlight_cb, False)
            ])

        self.main_actions.get_action('Undo').set_is_important(True)
        self.main_actions.get_action('Redo').set_is_important(True)
        self.main_actions.get_action('ShowPossible').set_is_important(True)
        self.main_actions.get_action('Tracker').set_is_important(True)

        self.uimanager.insert_action_group(self.main_actions, 0)
        self.uimanager.add_ui_from_string(self.ui)

    def setup_undo (self):
        self.cleared = [] # used for Undo memory
        self.cleared_notes = [] # used for Undo memory
        # Set up our UNDO stuff
        undo_widg = self.main_actions.get_action('Undo')
        redo_widg = self.main_actions.get_action('Redo')
        self.history = Undo.UndoHistoryList(undo_widg, redo_widg)
        for entry in self.gsd.__entries__.values():
            Undo.UndoableGenericWidget(entry, self.history,
                                       set_method = 'set_value_for_undo',
                                       get_method = 'get_value_for_undo',
                                       pre_change_signal = 'value-about-to-change'
                                       )
            Undo.UndoableGenericWidget(entry, self.history,
                                       set_method = 'set_notes_for_undo',
                                       get_method = 'get_notes_for_undo',
                                       signal = 'notes-changed',
                                       pre_change_signal = 'notes-about-to-change',
                                       )

    def setup_color (self):
        # setup background colors
        bgcol = self.settings.get_string('bg-color')
        if bgcol != '':
            self.gsd.set_bg_color(bgcol)

    def setup_autosave (self):
        GObject.timeout_add_seconds(self.settings.get_int('auto-save-interval') or 60, # in seconds...
                            self.autosave)

    def setup_main_boxes (self):
        self.vb = Gtk.VBox()
        # Add menu bar and toolbar...
        mb = self.uimanager.get_widget('/MenuBar')
        mb.show()
        self.vb.pack_start(mb, False, False, 0)
        self.tb = self.uimanager.get_widget('/Toolbar')
        self.tb.get_style_context().add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)
        self.vb.pack_start(self.tb, False, False, 0)
        self.main_area = Gtk.HBox()
        self.swallower = dialog_swallower.SwappableArea(self.main_area)
        self.swallower.show()
        self.vb.pack_start(self.swallower, True, True, 12)
        self.main_area.pack_start(self.gsd, True, True, 6)
        self.main_actions.set_visible(True)
        self.game_box = Gtk.VBox()
        self.main_area.show()
        self.vb.show()
        self.game_box.show()
        self.main_area.pack_start(self.game_box, False, False, 12)
        self.w.add(self.vb)

    def wrap_toggle (self, key_name, action):
        action.set_active(self.settings.get_boolean(key_name))
        action.connect('toggled', self.set_key, key_name)

    def set_key (self, action, key_name):
        self.settings.set_boolean(key_name, action.get_active())

    def setup_toggles (self):
        # sync up toggles with gsettings values...
        map(lambda tpl: self.wrap_toggle(*tpl),
            [('always-show-hints',
              self.main_actions.get_action('AlwaysShowPossible')),
             ('show-impossible-implications',
              self.main_actions.get_action('ShowImpossibleImplications')),
             ('show-toolbar',
              self.main_actions.get_action('ToggleToolbar')),
             ('highlight',
              self.main_actions.get_action('ToggleHighlight')),
             ('show-tracker',
              self.main_actions.get_action('Tracker')),
             ])

    @simple_debug
    def start_worker_thread (self, *args):
        n_new_puzzles = self.sudoku_maker.n_puzzles(new = True)
        if n_new_puzzles < self.settings.get_int('minimum-number-of-new-puzzles'):
            self.worker = threading.Thread(target = lambda *args: self.sudoku_maker.work(limit = 5))
            self.worker_connections = [
                self.timer.connect('timing-started', self.sudoku_maker.resume),
                self.timer.connect('timing-stopped', self.sudoku_maker.pause)
                ]
            self.worker.start()
        return True

    @simple_debug
    def stop_worker_thread (self, *args):
        if hasattr(self, 'worker'):
            self.sudoku_maker.stop()
            for c in self.worker_connections:
                self.timer.disconnect(c)

    def stop_dancer (self, *args):
        if hasattr(self, 'dancer'):
            self.dancer.stop_dancing()
            delattr(self, 'dancer')

    def start_dancer (self):
        import dancer
        self.dancer = dancer.GridDancer(self.gsd)
        self.dancer.start_dancing()

    @simple_debug
    def you_win_callback (self, grid):
        if hasattr(self, 'dancer'):
            return
        self.won = True
        # increase difficulty for next time.
        self.settings.set_double('difficulty', self.settings.get_double('difficulty') + 0.1)
        self.timer.finish_timing()
        self.sudoku_tracker.finish_game(self)
        if self.timer.active_time < 60:
            seconds = int(self.timer.active_time)
            sublabel = ngettext("You completed the puzzle in %d second",
                                "You completed the puzzle in %d seconds", seconds) % seconds
        elif self.timer.active_time < 3600:
            minutes = int(self.timer.active_time / 60)
            seconds = int(self.timer.active_time - minutes*60)
            minute_string = ngettext("%d minute", "%d minutes", minutes) % minutes
            second_string = ngettext("%d second", "%d seconds", seconds) % seconds
            sublabel = _("You completed the puzzle in %(minute)s and %(second)s") % {'minute': minute_string, 'second': second_string}
        else:
            hours = int(self.timer.active_time / 3600)
            minutes = int((self.timer.active_time - hours*3600) / 60)
            seconds = int(self.timer.active_time - hours*3600 - minutes*60)
            hour_string = ngettext("%d hour", "%d hours", hours) % hours
            minute_string = ngettext("%d minute", "%d minutes", minutes) % minutes
            second_string = ngettext("%d second", "%d seconds", seconds) % seconds
            sublabel = _("You completed the puzzle in %(hour)s, %(minute)s and %(second)s") % {'hour': hour_string, 'minute': minute_string, 'second': second_string}
        sublabel += "\n"
        sublabel += ngettext("You got %(n)s hint.", "You got %(n)s hints.", self.gsd.hints) % {'n':self.gsd.hints}
        sublabel += "\n"
        if self.gsd.impossible_hints:
            sublabel += ngettext("You had %(n)s impossibility pointed out.",
                                 "You had %(n)s impossibilities pointed out.",
                                 self.gsd.impossible_hints) % {'n':self.gsd.impossible_hints}
            sublabel += "\n"
        self.start_dancer()
        dialog_extras.show_message_dialog(_("You win!"), label = _("You win!"),
                                   sublabel = sublabel
                                   )

    @simple_debug
    @inactivate_new_game_etc
    def new_cb (self, *args):
        if (self.gsd.grid and self.gsd.grid.is_changed() and not self.won):
            try:
                if dialog_extras.show_boolean_dialog(
                    label = _("Save this game before starting new one?"),
                    custom_yes = _("_Save game for later"),
                    custom_no = _("_Abandon game"),
                    ):
                    self.save_game()
                else:
                    self.sudoku_tracker.abandon_game(self)
            except dialog_extras.UserCancelledError:
                # User cancelled new game
                return
        self.do_stop()
        self.select_game()


    @simple_debug
    def stop_game (self):
        if (self.gsd.grid
            and self.gsd.grid.is_changed()
            and (not self.won)):
            try:
                if dialog_extras.show_boolean_dialog(label = _("Save game before closing?")):
                    self.save_game(self)
            except dialog_extras.UserCancelledError:
                return
            self.do_stop()

    def do_stop (self):
        self.stop_dancer()
        self.gsd.grid = None
        self.tracker_ui.reset()
        self.history.clear()
        self.won = False
        self.old_tracker_view = None

    @simple_debug
    def resize_cb (self, widget, event, user_data=None):
        self.settings.set_int('width', event.width)
        self.settings.set_int('height', event.height)

    @simple_debug
    def quit_cb (self, *args):
        self.w.hide()
        if (self.gsd.grid
            and self.gsd.grid.is_changed()
            and (not self.won)):
            self.save_game(self)
        if Gtk.main_level() > 1:
            # If we are in an embedded mainloop, that means that one
            # of our "swallowed" dialogs is active, in which case we
            # have to quit that mainloop before we can quit
            # properly.
            if self.swallower.running:
                d = self.swallower.running
                d.response(Gtk.ResponseType.DELETE_EVENT)
            Gtk.main_quit() # Quit the embedded mainloop
            GObject.idle_add(self.quit_cb, 100) # Call ourselves again
                                               # to quit the main
                                               # mainloop
            return
        # make sure we really go away before doing our saving --
        # otherwise we appear sluggish.
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.stop_worker_thread()
        # allow KeyboardInterrupts, which calls quit_cb outside the main loop
        try:
            Gtk.main_quit()
        except RuntimeError:
            pass

    @simple_debug
    def save_game (self, *args):
        self.sudoku_tracker.save_game(self)

    def full_screen_cb (self, *args):
        if self.is_fullscreen:
            self.w.unfullscreen()
            self.is_fullscreen = False
        else:
            self.w.fullscreen()
            self.is_fullscreen = True

    @simple_debug
    def game_reset_cb (self, *args):
        clearer = Undo.UndoableObject(
            self.do_game_reset, #action
            self.undo_game_reset, #inverse
            self.history #history
            )
        clearer.perform()

    def do_game_reset (self, *args):
        self.gsd.cover_track()
        self.cleared.append(self.tinfo.save())
        self.cleared.append(self.gsd.reset_grid())
        self.cleared_notes.append((tracker_info.NO_TRACKER, self.gsd.clear_notes('All')))
        self.tinfo.reset()
        self.stop_dancer()

    def undo_game_reset (self, *args):
        self.tracker_ui.select_tracker(tracker_info.NO_TRACKER)
        for entry in self.cleared.pop():
            self.gsd.add_value(*entry)
        self.tinfo.load(self.cleared.pop())
        self.tracker_ui.select_tracker(self.tinfo.current_tracker)
        self.gsd.show_track()
        self.undo_clear_notes()

    def clear_top_notes_cb (self, *args):
        clearer = Undo.UndoableObject(
            lambda *args: self.do_clear_notes('Top'), #action
            self.undo_clear_notes, #inverse
            self.history
            )
        clearer.perform()

    def clear_bottom_notes_cb (self, *args):
        clearer = Undo.UndoableObject(
            lambda *args: self.do_clear_notes('Bottom'), #action
            self.undo_clear_notes, #inverse
            self.history
            )
        clearer.perform()

    def do_clear_notes(self, side):
        ''' Clear top, bottom, or all notes - in undoable fashion

        The side argument is used to specify which notes are to be cleared.
        'Top' - Clear only the top notes
        'Bottom' - Clear only the bottom notes
        '''
        self.cleared_notes.append((self.tinfo.current_tracker, self.gsd.clear_notes(side)))
        # Turn off auto-hint if the player clears the bottom notes
        if side == 'Bottom' and self.settings.get_boolean('always_show_hints'):
            always_show_hint_wdgt = self.main_actions.get_action('AlwaysShowPossible')
            always_show_hint_wdgt.activate()
        # Update the hints...in case we're redoing a clear of them
        if self.settings.get_boolean ('always_show_hints'):
            self.gsd.update_all_hints()

    def undo_clear_notes(self):
        ''' Undo previously cleared notes

        Clearing notes fills the cleared_notes list of notes that were cleared.
        '''
        cleared_tracker, cleared_notes = self.cleared_notes.pop()
        # Change the tracker selection if it was tracking during the clear
        if cleared_tracker != tracker_info.NO_TRACKER:
            self.tracker_ui.select_tracker(cleared_tracker)
        self.gsd.apply_notelist(cleared_notes)
        # Update the hints...in case we're undoing over top of them
        if self.settings.get_boolean('always-show-hints'):
            self.gsd.update_all_hints()
        # Redraw the notes
        self.gsd.update_all_notes()
        # Make sure we're still dancing if we undo after win
        if self.gsd.grid.check_for_completeness():
            self.start_dancer()

    @simple_debug
    def show_hint_cb (self, *args):
        self.gsd.show_hint()

    @simple_debug
    def auto_hint_cb (self, action, user_data=None):
        if action.get_active():
            self.gsd.always_show_hints = True
            self.gsd.update_all_hints()
        else:
            self.gsd.always_show_hints = False
            self.gsd.clear_notes('AutoHint')

    @simple_debug
    def impossible_implication_cb (self, action, user_data=None):
        if action.get_active():
            self.gsd.display_impossible_implications()
        else:
            self.gsd.hide_impossible_implications()

    @simple_debug
    def setup_tracker_interface (self):
        self.tracker_ui = TrackerBox(self)
        self.tracker_ui.show_all()
        self.tracker_ui.hide()
        self.tinfo = tracker_info.TrackerInfo()
        self.old_tracker_view = None
        self.game_box.add(self.tracker_ui)

    @simple_debug
    def tracker_toggle_cb (self, widg, user_data=None):
        if widg.get_active():
            if self.old_tracker_view:
                self.tinfo.set_tracker_view(self.old_tracker_view)
                self.tracker_ui.select_tracker(self.tinfo.current_tracker)
                self.gsd.show_track()
            self.tracker_ui.show_all()
        else:
            self.old_tracker_view = self.tinfo.get_tracker_view()
            self.tracker_ui.hide_tracker_cb(None)
            self.tracker_ui.hide()

    @simple_debug
    def toggle_toolbar_cb (self, widg, user_data=None):
        if widg.get_active():
            self.tb.show()
        else:
            self.tb.hide()

    def toggle_highlight_cb (self, widg, user_data=None):
        if widg.get_active():
            self.gsd.toggle_highlight(True)
        else:
            self.gsd.toggle_highlight(False)

    @simple_debug
    def show_info_cb (self, *args):
        if not self.gsd.grid:
            dialog_extras.show_message_dialog(parent = self.w,
                                       title = _("Puzzle Information"),
                                       label = _("There is no current puzzle.")
                                       )
            return
        puzzle = self.gsd.grid.virgin.to_string()
        diff = self.sudoku_maker.get_difficulty(puzzle)
        information = _("Calculated difficulty: ")
        try:
            information += {'easy': _('Easy'),
                            'medium': _('Medium'),
                            'hard': _('Hard'),
                            'very hard': _('Very Hard')}[diff.value_category()]
        except KeyError:
            information += diff.value_category()
        information += " (%1.2f)" % diff.value
        information += "\n"
        information += _("Number of moves instantly fillable by elimination: ")
        information += str(int(diff.instant_elimination_fillable))
        information += "\n"
        information += _("Number of moves instantly fillable by filling: ")
        information += str(int(diff.instant_fill_fillable))
        information += "\n"
        information += _("Amount of trial-and-error required to solve: ")
        information += str(len(diff.guesses))
        dialog_extras.show_message_dialog(parent = self.w,
                                   title = _("Puzzle Statistics"),
                                   label = _("Puzzle Statistics"),
                                   sublabel = information)

    @simple_debug
    def autosave (self):
        # this is called on a regular loop and will autosave if we
        # have reason to...
        if self.gsd.grid and self.gsd.grid.is_changed() and not self.won:
            self.sudoku_tracker.save_game(self)
        return True

    @simple_debug
    def show_about (self, *args):
        about = Gtk.AboutDialog(
                    transient_for=self.w,
                    program_name=APPNAME,
                    version=VERSION,
                    copyright=COPYRIGHT,
                    license_type=Gtk.License.GPL_2_0,
                    comments=DESCRIPTION,
                    authors=AUTHORS,
                    website=WEBSITE,
                    website_label=WEBSITE_LABEL,
                    logo_icon_name="gnome-sudoku",
                    translator_credits=_("translator-credits"))
        about.connect("response", lambda d, r: d.destroy())
        about.show()

    @simple_debug
    def show_help (self, *args):
        try:
            Gtk.show_uri(self.w.get_screen(), "help:gnome-sudoku", Gtk.get_current_event_time())
        except GObject.GError, error:
            # FIXME: This should create a pop-up dialog
            print _('Unable to display help: %s') % str(error)

    @simple_debug
    def print_game (self, *args):
        printing.print_sudokus([self.gsd], self.w)

    @simple_debug
    def print_multiple_games (self, *args):
        gp = printing.GamePrinter(self.sudoku_maker)
        gp.run_dialog()

class TrackerBox (Gtk.VBox):

    @simple_debug
    def __init__ (self, main_ui):
        GObject.GObject.__init__(self)
        self.footprint_img =  cairo.ImageSurface.create_from_png(
                                    os.path.join(IMAGE_DIR, "footprints.png"));
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain(DOMAIN)
        self.builder.add_from_file(os.path.join(UI_DIR, 'tracker.ui'))
        self.main_ui = main_ui
        self.tinfo = tracker_info.TrackerInfo()
        self.tinfo.ui = self
        self.vb = self.builder.get_object('vbox1')
        self.vb.unparent()
        self.pack_start(self.vb, True, True, 0)
        self.setup_actions()
        self.setup_tree()
        self.show_all()

    @simple_debug
    def reset (self):

        for tree in self.tracker_model:
            if tree[0] > -1:
                self.tracker_model.remove(tree.iter)
        self.tinfo.reset()
        self.tracker_actions.set_sensitive(False)

    @simple_debug
    def setup_tree (self):
        self.tracker_tree = self.builder.get_object('TrackerTreeView')
        self.tracker_model = Gtk.ListStore(int, GdkPixbuf.Pixbuf, str)
        self.tracker_model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
        self.tracker_tree.set_model(self.tracker_model)
        col1 = Gtk.TreeViewColumn("", Gtk.CellRendererPixbuf(), pixbuf = 1)
        rend = Gtk.CellRendererText()
        col2 = Gtk.TreeViewColumn("", rend, text = 2)
        col2.set_cell_data_func(rend, self.draw_tracker_name)
        self.tracker_tree.append_column(col2)
        self.tracker_tree.append_column(col1)
        # Our initial row...
        pixbuf = self.get_tracker_pixbuf(
            (0, 0, 0)
            )
        self.tracker_model.append([-1, pixbuf, _('Untracked')])
        self.tracker_tree.get_selection().connect('changed', self.selection_changed_cb)

    @simple_debug
    def setup_actions (self):
        self.tracker_actions = Gtk.ActionGroup('tracker_actions')
        self.tracker_actions.add_actions(
            [('Remove',
              Gtk.STOCK_CLEAR,
              _('_Remove'),
              None, _('Delete selected tracker.'),
              self.remove_tracker_cb
              ),
             ('Hide',
              Gtk.STOCK_CLEAR,
              _('H_ide'),
              None, _('Hide current tracker entries.'),
              self.hide_tracker_cb
              ),
             ('Apply',
              Gtk.STOCK_APPLY,
              _('A_pply'),
              None, _('Apply all tracked values and remove the tracker.'),
              self.apply_tracker_cb
              ),
             ]
            )
        o = self.builder.get_object('RemoveTrackerButton')
        o.set_related_action(self.tracker_actions.get_action('Remove'))

        o = self.builder.get_object('HideTrackerButton')
        o.set_related_action(self.tracker_actions.get_action('Hide'))

        o = self.builder.get_object('ApplyTrackerButton')
        o.set_related_action(self.tracker_actions.get_action('Apply'))

        self.builder.get_object('AddTrackerButton').connect('clicked',
                                                          self.add_tracker)
        # Default to insensitive (they only become sensitive once a tracker is added)
        self.tracker_actions.set_sensitive(False)

    def draw_tracker_name(self, column, cell, model, iter, user_data=None):
        if model.get_value(iter, 0) == self.tinfo.showing_tracker:
            cell.set_property('weight', Pango.Weight.BOLD)
        else:
            cell.set_property('weight', Pango.Weight.NORMAL)

    @simple_debug
    def add_tracker (self, *args, **keys):
        if keys and keys.has_key('tracker_id'):
            tracker_id = self.tinfo.create_tracker(keys['tracker_id'])
        else:
            tracker_id = self.tinfo.create_tracker()
        pixbuf = self.get_tracker_pixbuf(
            self.tinfo.get_color(tracker_id)
            )
        # select our new tracker
        self.tracker_tree.get_selection().select_iter(
            self.tracker_model.append([tracker_id,
                                  pixbuf,
                                  _("Tracker %s") % (tracker_id + 1)]
                                  )
            )
        self.tinfo.set_tracker(tracker_id)

    @simple_debug
    def get_tracker_pixbuf (self, color):
        surface = cairo.ImageSurface (cairo.FORMAT_ARGB32, 64, 64)
        cr = cairo.Context(surface)
        cr.set_source_rgb(*color);
        cr.mask_surface(self.footprint_img, 0, 0)
        cr.fill()

        return Gdk.pixbuf_get_from_surface(surface, 0, 0, 64, 64)

    @simple_debug
    def find_tracker (self, tracker_id):
        for row in self.tracker_model:
            if row[0] == tracker_id:
                return row
        return None

    @simple_debug
    def select_tracker (self, tracker_id):
        track_row = self.find_tracker(tracker_id)
        if track_row:
            self.tracker_tree.get_selection().select_iter(track_row.iter)
            self.tinfo.set_tracker(tracker_id)

    def redraw_row(self, tracker_id):
        track_row = self.find_tracker(tracker_id)
        if track_row:
            self.tracker_model.row_changed(self.tracker_model.get_path(track_row.iter), track_row.iter)

    def set_tracker_action_sense(self, enabled):
        self.tracker_actions.set_sensitive(True)
        for action in self.tracker_actions.list_actions():
            action.set_sensitive(self.tinfo.showing_tracker != tracker_info.NO_TRACKER)

    @simple_debug
    def selection_changed_cb (self, selection, user_data=None):
        mod, itr = selection.get_selected()
        if itr:
            selected_tracker_id = mod.get_value(itr, 0)
        else:
            selected_tracker_id = tracker_info.NO_TRACKER
        if selected_tracker_id != tracker_info.NO_TRACKER:
            self.main_ui.gsd.cover_track()
        # Remove the underline on the showing_tracker
        self.redraw_row(self.tinfo.showing_tracker)
        self.tinfo.set_tracker(selected_tracker_id)
        self.set_tracker_action_sense(self.tinfo.showing_tracker != tracker_info.NO_TRACKER)
        # Show the tracker
        if selected_tracker_id != tracker_info.NO_TRACKER:
            self.main_ui.gsd.show_track()
        self.main_ui.gsd.update_all_notes()
        if self.main_ui.settings.get_boolean('always-show-hints'):
            self.main_ui.gsd.update_all_hints()

    @simple_debug
    def remove_tracker_cb (self, action, user_data=None):
        mod, itr = self.tracker_tree.get_selection().get_selected()
        # This should only be called if there is an itr, but we'll
        # double-check just in case.
        if itr:
            clearer = Undo.UndoableObject(
                self.do_delete_tracker,
                self.undo_delete_tracker,
                self.main_ui.history
                )
            clearer.perform()

    @simple_debug
    def hide_tracker_cb (self, action, user_data=None):
        hiding_tracker = self.tinfo.showing_tracker
        self.select_tracker(tracker_info.NO_TRACKER)
        self.main_ui.gsd.cover_track(True)
        self.main_ui.gsd.update_all_notes()
        self.set_tracker_action_sense(False)
        self.redraw_row(hiding_tracker)
        self.redraw_row(tracker_info.NO_TRACKER)

    @simple_debug
    def apply_tracker_cb (self, action, user_data=None):
        '''Apply Tracker button action
        '''
        # Shouldn't be here if no tracker is showing
        if self.tinfo.showing_tracker == tracker_info.NO_TRACKER:
            return
        # Apply the tracker in undo-able fashion
        applyer = Undo.UndoableObject(
            self.do_apply_tracker,
            self.undo_apply_tracker,
            self.main_ui.history
            )
        applyer.perform()

    def do_apply_tracker(self):
        '''Apply the showing tracker to untracked

        All of the values and notes will be transferred to untracked and
        the tracker is deleted.
        '''
        track_row = self.find_tracker(self.tinfo.showing_tracker)
        if not track_row:
            return
        # Delete the tracker
        cleared_values, cleared_notes = self.do_delete_tracker(True)
        # Apply the values
        for x, y, val, tid in cleared_values:
            self.main_ui.gsd.set_value(x, y, val)
        # Then apply the notes
        self.main_ui.gsd.apply_notelist(cleared_notes, True)
        # Store the undo counts
        self.main_ui.cleared.append(len(cleared_values))
        self.main_ui.cleared_notes.append(len(cleared_notes))

    def undo_apply_tracker(self):
        '''Undo a previous tracker apply

        The number of cleared values and notes are stored during the apply.
        The undo is called for each of them, then the tracker delete is
        undone.
        '''
        # Undo all of the applied values and notes
        value_count = self.main_ui.cleared.pop()
        note_count = self.main_ui.cleared_notes.pop()
        count = 0
        while count < (value_count + note_count):
            self.main_ui.history.undo()
            count += 1
        # Undo the tracker delete
        self.undo_delete_tracker()

    def do_delete_tracker(self, for_apply = False):
        '''Delete the current tracker
        '''
        track_row = self.find_tracker(self.tinfo.showing_tracker)
        if not track_row:
            return
        ui_row = [track_row[0], track_row[1], track_row[2]]
        # For the values, store it like (tracker_id, list_of_cleared_values)
        cleared_values = self.main_ui.gsd.delete_by_tracker()
        self.main_ui.cleared.append((self.tinfo.showing_tracker, ui_row, cleared_values))
        # The notes already have tracker info in them, so just store the list
        cleared_notes = self.main_ui.gsd.clear_notes(tracker = self.tinfo.showing_tracker)
        self.main_ui.cleared_notes.append(cleared_notes)
        # Delete it from tracker_info
        self.hide_tracker_cb(None)
        self.tracker_model.remove(track_row.iter)
        self.tinfo.delete_tracker(ui_row[0])
        # Return all of the data for "Apply Tracker" button
        if for_apply:
            return (cleared_values, cleared_notes)

    def undo_delete_tracker(self):
        '''Undo a tracker delete
        '''
        # Values are stored like (tracker_id, list_of_cleared_values)
        tracker_id, ui_row, cleared_values = self.main_ui.cleared.pop()
        # Recreate it in tracker_info
        self.tinfo.create_tracker(tracker_id)
        # Add it to the tree
        self.tracker_tree.get_selection().select_iter(self.tracker_model.append(ui_row))
        # Add all the values
        for value in cleared_values:
            self.main_ui.gsd.add_value(*value)
        # The notes already have tracker info in them, so just store the list
        self.main_ui.gsd.apply_notelist(self.main_ui.cleared_notes.pop())

def start_game ():
    if options.debug:
        print 'Starting GNOME Sudoku in debug mode'

    ##  You must call g_thread_init() before executing any other GLib
    ##  functions in a threaded GLib program.
    GObject.threads_init()

    if options.profile:
        options.profile = False
        profile_me()
        return

    u = UI()
    if not u.quit:
        try:
            Gtk.main()
        except KeyboardInterrupt:
            # properly quit on a keyboard interrupt...
            u.quit_cb()

def profile_me ():
    print 'Profiling GNOME Sudoku'
    import tempfile, hotshot, hotshot.stats
    pname = os.path.join(tempfile.gettempdir(), 'GNOME_SUDOKU_HOTSHOT_PROFILE')
    prof = hotshot.Profile(pname)
    prof.runcall(start_game)
    stats = hotshot.stats.load(pname)
    stats.strip_dirs()
    stats.sort_stats('time', 'calls').print_stats()

